/***************************************************************************
 * amx_ejl_missiles.sma      version 3.7.4             Date: 6/18/2003
 *  Author: Eric Lidman      ejlmozart@hotmail.com
 *  Alias: Ludwig van        Upgrade: http://lidmanmusic.com/cs/plugins.html           
 *
 *  Add missiles to Counter-Strike. Fire a variety of types of missiles
 *   including laser guided missiles, heat seeking missiles, and missiles 
 *   with guncam on so you see what the missile sees and control it thusly.
 *   Missiles can either be bought or given free to clients and there are 
 *   several options for this. In game menus and help pages are included.
 *   Note: Heatseeking missiles only seek running jetpacks, from the plugins
 *   "Jetpack Model No. Lud1337" and "Jetpack". You must have the lastest
 *   version of whichever jetpack plugin for it to work. In addition,
 *   missiles can be shot down with common guns or with the laser from my 
 *   "CS Laser Guns and Laser Stats" plugin, you need the latest lasers 
 *   plugin 1.7.1 for that to work. If you successfully shoot down a 
 *   missile with a laser, you get a prize of free missiles for the 
 *   remainder of the round plus extra missiles for the next round. Lastly, 
 *   you need the vexd utilities module properly installed in order for 
 *   this plugin to work at all. See:
 *
 *     Lasers:       http://amxmod.net/forums/viewtopic.php?t=9653
 *     Jetpack:      http://amxmod.net/forums/viewtopic.php?t=14196
 *     Vexd Module:  http://amxmod.net/forums/viewtopic.php?t=16594
 *     Cmath Module: http://amxmod.net/forums/viewtopic.php?t=14037
 *
 *  PODBOT: If using my podbots controller, podbots use this plugin too. 
 *
 *  Admin Commands:
 *
 *    amx_missiles             -- toggles on and off missiles enabled
 *    amx_missilesbuying       -- toggles on/off missile buy requirements
 *    amx_missiles_ammo1       --# of guncam missiles given per round  
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo2       --# of laserguide missiles given per round 
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo3       --# of heatseek missiles given per round  
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo4       --# of common missiles given per round  
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo5       --# of anti-missile missiles given per round  
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo6       --# of joke missiles --(attacks luds rope)
 *                             free to clients if missile buying is off
 *                             (for admins without amx_cvar access)
 *    amx_missiles_ammo7       --# of swirling death missiles given
 *                             per round free to clients if missile buying
 *                             is off. Only applicable if you are using 
 *                             the cmath module
 *                             (for admins without amx_cvar access)
 *    amx_missile1 <speed=1000> <chicken=0> <deadly=0> : guncam style 
 *    amx_missile2 <speed=1000> <chicken=0> <deadly=0> : laserguided style
 *    amx_missile3 <speed=1000> <chicken=0> <deadly=0> : heatseeker style
 *    amx_missile4 <speed=1000> <chicken=0> <deadly=0> : common missile
 *    amx_missile7 <speed=1000> <chicken=0> <deadly=0> : swirling death
 *
 *   Examples:
 * 
 *    amx_missile2 2000 0 0   --fires fast non-lethal laserguided missile
 *    amx_missile3 500 1 1    --fires slow lethal jetpack seeking chicken 
 *
 *  Client Commands:
 *
 *    amx_missile              -- fires common missile
 *    amx_laserguided_missile  -- fires missile seeking red dot in aim
 *    amx_heatseeking_missile  -- seeks nearest running jetpack
 *    amx_guncamera_missile    -- fires missile you control in camera veiw
 *    amx_anti_missile         -- activates antimissile system, aim at
 *                                missile you want to destroy, if it sees
 *                                the missile in your aim, it automatically
 *                                launches a missile interceptor at it
 *    amx_ropeseeking_missile  -- seeks nearest ninja-rope (luds only)
 *    amx_swirlingdeath_missile - a central missile with ring of 6 (default)
 *                                missiles rotating around it. When missiles
 *                                run out of fuel, they break formation and
 *                                cause massive destruction.
 *    amx_sdmf <fuel>          -- client uses this to set amount of time 
 *                                swirling death missile flies until it
 *                                runs out of fuel. Only works if cvar
 *                                amx_missile_allow_sdmf is 1. Default: 0  
 *    missile_menu             -- opens the client missile menu
 *    say /missile             -- opens the client missile menu
 *    say /missile_help        -- show window with info about missles
 *
 *  CVARs:  put in your admin.cfg for your defaults
 *
 *    amx_luds_missiles 1       --0 to disable missiles
 *    amx_missile_buy 0         --1 requires missiles to be purchased
 *    amx_missile_cost1 3000    --ammount of money taken each time you fire
 *                                a guncamera missile (only if buying is on)
 *    amx_missile_cost2 3000    --ammount of money taken each time you fire
 *                                a laserguide missile(only if buying is on)
 *    amx_missile_cost3 4000    --ammount of money taken each time you fire
 *                                a heatseek missile (only if buying is on)
 *    amx_missile_cost4 1000    --ammount of money taken each time you fire
 *                                a common missile (only if buying is on)
 *    amx_missile_cost5 2000    --ammount of money taken each time you fire
 *                                an anti-missile (only if buying is on)
 *    amx_missile_cost6 4000    --ammount of money taken each time you fire
 *                                a ropeseek missile (only if buying is on)
 *    amx_missile_cost7 5000    --ammount of money taken each time you fire
 *                                a swirling death missile (only if buying
 *                                is on and only if you use cmath module)
 *    amx_missile_ammo1 1       --# of guncam missiles given per round  
 *                                free to clients if amx_missile_buy is 0
 *    amx_missile_ammo2 1       --# of laserguide missiles given per round 
 *                                free to clients if amx_missile_buy is 0
 *    amx_missile_ammo3 1       --# of heatseek missiles given per round  
 *                                free to clients if amx_missile_buy is 0
 *    amx_missile_ammo4 2       --# of common missiles given per round  
 *                                free to clients if amx_missile_buy is 0
 *    amx_missile_ammo5 1       --# of anti-missile missiles given per round  
 *                                free to clients if amx_missile_buy is 0
 *    amx_missile_ammo6 1       --# of ropeseeking missiles ... joke on luds
 *                                server given per round free to clients if
 *                                amx_missile_buy is 0
 *    amx_missile_ammo7 1       --# of swirling death missiles given
 *                                per round free to clients if missile buying
 *                                is 0. Only applicable if you are using 
 *                                the cmath module
 *	amx_missile_ammo7takeall 0 -if set to 1, this cvar causes swirling 
 *                                death missile to use 7 missiles in the 
 *                                player's missile inventory draw from all
 *                                types of missiles instead of it having its
 *                                own indepedent inventory count. It makes
 *                                the massively destructive swirling death 
 *                                missiles more "costly". 
 *    amx_missile_speed 1000    --sets the default speed of missiles for
 *                                clients and for admins when no speed is
 *                                specified in the missile command
 *                                Sets all but the swirling death speed
 *    amx_missile_rsspeed 1400  --sets the speed of ropeseeking missiles
 *    amx_missile_hsspeed 1100  --sets the speed of heatseeking missiles
 *    amx_missile_fuel 6        --number of seconds a missile is driven 
 *                                before it falls to the ground out of fuel
 *    amx_missile_sdfuel 2.0    --number of seconds a swirling death missile
 *                                is driven before it "mirvs" or breaks then
 *                                falls to the ground out of fuel
 *    amx_missile_sdspeed 750   --sets the speed of swirling death missiles
 *    amx_missile_sdcount 6     --sets the number missiles in swirling death
 *    amx_missile_sdrotate 6    --sets the rotation speed of swirling death
 *    amx_missile_sdradius 32   --sets the radius of swirling death missiles
 *    amx_missile_allow_sdmf 0  --sets whether clients are allowed to set
 *                                thier own fuel amounts for swirling death
 *                                missile. Default 0. Enable with 1 
 *    amx_missile_obeygravity 1 --makes missile obey server gravity rules,
 *                                set to 0 for missiles that travel straight
 *                                because they are not affected by gravity
 *    amx_missile_obeyffcvar 1  --makes missile obey server friendly fire
 *                                rules. Set this to 0 if you want to have
 *                                missiles be non-friendly fire even if
 *                                friendly fire (mp_friendlyfire 1) is on.
 *    amx_missile_damradius1 90 --the first sphere of damage zone around a
 *                                missile impact, used in conjunction with 
 *                                the cvar amx_missile_radiusdam1
 *    amx_missile_damradius2 160 -the second sphere of damage zone around a
 *                                missile impact, used in conjunction with 
 *                                the cvar amx_missile_radiusdam2
 *    amx_missile_damradius3 270 -the third sphere of damage zone around a
 *                                missile impact, used in conjunction with 
 *                                the cvar amx_missile_radiusdam3
 *    amx_missile_radiusdam1 100 -the amount of damage caused by a missile
 *                                in its 1st damage sphere as defined by 
 *                                the cvar amx_damradius 1
 *                                (note: if any radiusdam cvar is set to
 *                                100 or more it is considered a headshot
 *                                and takes all health regardless of how
 *                                much health the victim has)
 *    amx_missile_radiusdam2 66 --the amount of damage caused by a missile
 *                                in its 1st damage sphere as defined by 
 *                                the cvar amx_damradius 2
 *    amx_missile_radiusdam3 25 --the amount of damage caused by a missile
 *                                in its 1st damage sphere as defined by 
 *                                the cvar amx_damradius 3
 *    amx_missile_tkpunish1 1  -- for friendlyfire on, the option to punish
 *                                team killer. Quota is from the cvar 
 *                                amx_missile_tkpunsish2. options:
 *                                      0 = no action on team killer
 *                                      1 = kick tker on tk quota
 *                                      2 = 3 hour ban tker on quota
 *                                      3 = always kill tker, kick on quota
 *                                      4 = always kill tker, ban on quota
 *    amx_missile_tkpunish2 3  -- for friendlyfire on, quantity of teammates
 *                                a player can kill before a kick or ban can 
 *                                result depending on the above cvar 
 *                                amx_missile_tkpunish1 is set
 *    amx_missile_prizes 5      --if in the rare event a player shoots down
 *                                a missile belonging to an opposing player, 
 *                                he gets a prize, free missiles. Cvar sets
 *                                how many of each type of missile are given
 *   	amx_missile_radarbattery  --sets the amount of time a player can use 
 *                                his anti-missile radar per round. def: 100
 *    amx_missile_botsnolimit 1 --since bots are stupid and waste their
 *                                missiles, we give them a handicap with this
 *                                cvar set to 1 so that bots get an unlimited
 *                                quantity of missiles
 *    amx_missile_atspawn 0     --this cvar limits the two types of missiles
 *                                responsible for spawn rape, guncamera and
 *                                swirling death, from being fired until 15
 *                                seconds of a round has passed. Set cvar to
 *                                1 to allow those missiles to be fired with-
 *                                out being limited by round start
 *
 *  Additional:
 *
 *    This plugin logs missile kills in the standard HL format so that stats
 *    parsing programs like psychostats will record missile kills. If you
 *    need a missile picture for the stats pages, write to author or better
 *    yet, go to the official amxmod site forum post for this plugin:
 *
 *    http://amxmod.net/forums/viewtopic.php?t=16561 
 *
 *    If you have the module from SpaceDude's warcraft3 plugin (war3_ms) you
 *    can make the deahtmessages look more real, so laser kills look like an
 *    actual player killing another player instead of a suicide on the part
 *    of the killed player. To enable this, you need to change the code
 *    below. Simply change the value to 1 where it has #define use_newdeathmsg 0
 *    Its simple, a one line change, thats it, compile and its ready to go.
 *
 *    Big huge thanks to SpaceDude for the swirling death missile's math.
 *    He basically made swirling death, sent it to me, and I put it in here.
 *
 **************************************************************************/

/*  To get the better deathmessages. Change this one  */
/*  line so it looks like:    #define use_newdeathmsg 1      */

#define use_newdeathmsg 0


/************************************************************************************
 *                                                                                  *
 *  *end* customizable section of code. other changes can be done with the cvars    *
 *                                                                                  *
 ************************************************************************************/


#include <amxmod> 
#include <amxmisc>
#include <fun>
#include <Vexd_Utilities>
#if use_newdeathmsg == 1
new gmsgDeathMsg
new redundant_plug
#endif

new Float:g_fAngle 
#define DT 0.1 
#define PI 3.1415926535897932384626433832795 


new beam, boom, ls_dot
new bool:cstrike_running
new bool:dod_running
new dod_team[33]
new DoOnce
new round_st
new has_rocket[33]
/* missile_inv:  fake,guncam,laserguide,heatseeker,normal,antimissile,ropeseeking,multimissile */
new missile_inv[33][8] 
new missile_win[33]
new using_menu[33]
new is_scan_rocket[33]
new is_heat_rocket[33]
new radar_batt[33]
new tkcount[33]
new Float:user_sdmf[33]
new cmd_jetpackrun[33]
new cmd_onrope[33]

public client_connect(id){
	dod_team[id] = 0
	is_scan_rocket[id] = 0
	is_heat_rocket[id] = 0
	using_menu[id] = 0
	has_rocket[id] = 0
	missile_win[id] = 0
	if(get_cvar_num("amx_missile_buy") == 0){
		missile_inv[id][1] = get_cvar_num("amx_missile_ammo1")
		missile_inv[id][2] = get_cvar_num("amx_missile_ammo2")
		missile_inv[id][3] = get_cvar_num("amx_missile_ammo3")
		missile_inv[id][4] = get_cvar_num("amx_missile_ammo4")
		missile_inv[id][5] = get_cvar_num("amx_missile_ammo5")
		missile_inv[id][6] = get_cvar_num("amx_missile_ammo6")
		missile_inv[id][7] = get_cvar_num("amx_missile_ammo7")
	}
	radar_batt[id] = get_cvar_num("amx_missile_radarbattery")
	tkcount[id] = 0
	user_sdmf[id] = 0.0
	cmd_jetpackrun[id] = 0
	cmd_onrope[id] = 0

	if(get_cvar_num("amx_missile_conmsg")){
		if(get_cvar_num("amx_luds_missiles") == 0)
			client_cmd(id,"echo [AMX]  Missiles are DISABLED right now.") 
		else
			client_cmd(id,"echo [AMX]  Missiles are ENABLED right now.")
	}

	return PLUGIN_CONTINUE
}

public client_disconnect(id){
	dod_team[id] = 0
	remove_task(35632+id)
	is_scan_rocket[id] = 0
	is_heat_rocket[id] = 0
	has_rocket[id] = 0
	tkcount[id] = 0
	user_sdmf[id] = 0.0
	cmd_jetpackrun[id] = 0
	cmd_onrope[id] = 0
	return PLUGIN_CONTINUE
}

public admin_missiles(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	if(get_cvar_num("amx_luds_missiles") == 0){
		set_cvar_string("amx_luds_missiles","1")
		client_print(0,print_chat,"[AMX]  Admin has enabled missiles") 
		console_print(id,"[AMX]  You have enabled missiles")
	}else {
		set_cvar_string("amx_luds_missiles","0")
		client_print(0,print_chat,"[AMX]  Admin has disabled missiles") 
		console_print(id,"[AMX]  You have disabled missiles")
	}	
	new authid[16],name[32]
	get_user_authid(id,authid,16)
	get_user_name(id,name,32)
	log_to_file("addons/amx/admin.log","^"%s<%d><%s><>^" missiles",name,get_user_userid(id),authid)

	return PLUGIN_HANDLED
}

public admin_missilebuy(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	if(get_cvar_num("amx_missile_buy") == 0){
		set_cvar_string("amx_missile_buy","1")
		client_print(0,print_chat,"[AMX]  Admin requires you pay for your your missiles now.") 
		console_print(id,"[AMX]  You have required players pay for missiles")
	}else {
		set_cvar_string("amx_missile_buy","0")
		client_print(0,print_chat,"[AMX]  Admin has decreed that missiles are free now")  
		console_print(id,"[AMX]  You have made missiles free")
	}	
	new authid[16],name[32]
	get_user_authid(id,authid,16)
	get_user_name(id,name,32)
	log_to_file("addons/amx/admin.log","^"%s<%d><%s><>^" missiles_buying",name,get_user_userid(id),authid)

	return PLUGIN_HANDLED
}

public admin_missile_ammo(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED
	if(get_cvar_num("amx_missile_buy") == 1){
		console_print(id,"[AMX]  For this command to have effect, disable missile buying first. amx_missilesbuying")
		return PLUGIN_HANDLED
	}  
	new cmd[32],arg[10]
	read_argv(0,cmd,31)
	read_argv(1,arg,9)
	new quantity = strtonum(arg) 
	if(equal(cmd[17],"1",1)){
		set_cvar_num("amx_missile_ammo1",quantity)
		console_print(id,"[AMX] Effective next round, # of free guncam missiles set to %d",quantity)
	}
	else if(equal(cmd[17],"2",1)){
		set_cvar_num("amx_missile_ammo2",quantity)
		console_print(id,"[AMX] Effective next round, # of free laserguided missiles set to %d",quantity)
	}
	else if(equal(cmd[17],"3",1)){
		set_cvar_num("amx_missile_ammo3",quantity)
		console_print(id,"[AMX] Effective next round, # of free heatseeking missiles set to %d",quantity)
	}
	else if(equal(cmd[17],"4",1)){
		set_cvar_num("amx_missile_ammo4",quantity)
		console_print(id,"[AMX] Effective next round, # of free common missiles set to %d",quantity)
	}
	else if(equal(cmd[17],"5",1)){
		set_cvar_num("amx_missile_ammo5",quantity)
		console_print(id,"[AMX] Effective next round, # of free anti missile shots set to %d",quantity)
	}
	else if(equal(cmd[17],"6",1)){
		set_cvar_num("amx_missile_ammo6",quantity)
		console_print(id,"[AMX] Effective next round, # of free ropeseeking shots set to %d",quantity)
	}
	else if(equal(cmd[17],"7",1)){
		set_cvar_num("amx_missile_ammo7",quantity)
		console_print(id,"[AMX] Effective next round, # of free swirling death missiles set to %d",quantity)
	}
	new authid[16],name[32]
	get_user_authid(id,authid,16)
	get_user_name(id,name,32)
	log_to_file("addons/amx/admin.log","^"%s<%d><%s><>^" %s %d",name,get_user_userid(id),authid,cmd,quantity)
	return PLUGIN_HANDLED
}

public vexd_pfntouch(pToucher, pTouched) {
	new szClassName[32]
	Entvars_Get_String(pToucher, EV_SZ_classname, szClassName, 31)
	if(equal(szClassName, "lud_missile")) {
		new radius1 = get_cvar_num("amx_missile_damradius1")
		new radius2 = get_cvar_num("amx_missile_damradius2")
		new radius3 = get_cvar_num("amx_missile_damradius3")
		new dam1 = get_cvar_num("amx_missile_radiusdam1")
		new dam2 = get_cvar_num("amx_missile_radiusdam2")
		new dam3 = get_cvar_num("amx_missile_radiusdam3")
		remove_task(2020+pToucher)
		emit_sound(pToucher, CHAN_WEAPON, "weapons/explode3.wav", 1.0, ATTN_NORM, 0, PITCH_NORM) 
		emit_sound(pToucher, CHAN_VOICE, "weapons/explode3.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
		new Float:fl_vExplodeAt[3]
		Entvars_Get_Vector(pToucher, EV_VEC_origin, fl_vExplodeAt)
		new vExplodeAt[3]
		vExplodeAt[0] = floatround(fl_vExplodeAt[0])
		vExplodeAt[1] = floatround(fl_vExplodeAt[1])
		vExplodeAt[2] = floatround(fl_vExplodeAt[2])
		new id = Entvars_Get_Edict(pToucher, EV_ENT_owner)
		new unarmed = missile_inv[id][0]
		AttachView(id, id)
		if(has_rocket[id] == pToucher)
			has_rocket[id] = 0
		new origin[3]
		new maxplayers = get_maxplayers()+1
		for (new i=1; i<=maxplayers; i++) {
			if(DoOnce == 0){
				if((cvar_exists("mp_friendlyfire")) && (unarmed !=2) ){
					if( (get_cvar_num("mp_friendlyfire") == 1) && (get_cvar_num("amx_missile_obeyffcvar") == 1) ){
						if((is_user_alive(i) == 1) && (i != id)){
							get_user_origin(i,origin)
							new dist = get_distance(origin,vExplodeAt)
							if(dod_running){
								if(dod_team[i] != dod_team[id]){
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,0)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,0)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}else{
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,1)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,1)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}
							}else{
								if(get_user_team(i) != get_user_team(id)){
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,0)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,0)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}else{
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,1)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,1)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}
							}
						}				
					}else{
						if(dod_running){
							if(dod_team[i] != dod_team[id]){
								if((is_user_alive(i) == 1) && (i != id)){
									get_user_origin(i,origin)
									new dist = get_distance(origin,vExplodeAt)
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,0)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,0)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}
							}
						}else{
							if(get_user_team(i) != get_user_team(id)){
								if((is_user_alive(i) == 1) && (i != id)){
									get_user_origin(i,origin)
									new dist = get_distance(origin,vExplodeAt)
									if(dist < radius1)
										do_victim(i,id,dam1,unarmed,0)
									else if(dist < radius2)
										do_victim(i,id,dam2,unarmed,0)
									else if(dist < radius3)
										do_victim(i,id,dam3,unarmed,0)
								}
							}
						}
					}
				}else{
					if((is_user_alive(i) == 1) && (i != id)){
						get_user_origin(i,origin)
						new dist = get_distance(origin,vExplodeAt)
						if(dist < radius1)
							do_victim(i,id,1,unarmed,0)
						else if(dist < radius2)
							do_victim(i,id,0,unarmed,0)
						else if(dist < radius3)
							do_victim(i,id,0,unarmed,0)
					}
				}
			}
		}
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(3)
		write_coord(vExplodeAt[0])
		write_coord(vExplodeAt[1])
		write_coord(vExplodeAt[2])
		write_short(boom)
		write_byte(100)
		write_byte(15)
		write_byte(0)
		message_end()
		RemoveEntity(pToucher)
		is_heat_rocket[id] = 0
		is_scan_rocket[id] = 0

		new szClassName1[32]
		if(pTouched > maxplayers)
			Entvars_Get_String(pTouched, EV_SZ_classname, szClassName1, 31)
		if(equal(szClassName1, "lud_missile")) {
			remove_task(2020+pTouched)
			emit_sound(pTouched, CHAN_WEAPON, "weapons/explode3.wav", 1.0, ATTN_NORM, 0, PITCH_NORM) 
			emit_sound(pTouched, CHAN_VOICE, "weapons/explode3.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
			new id2 = Entvars_Get_Edict(pTouched, EV_ENT_owner)
			AttachView(id2, id2)
			if(has_rocket[id2] == pTouched){
				has_rocket[id2] = 0
				is_heat_rocket[id2] = 0
				is_scan_rocket[id2] = 0
			}
			RemoveEntity(pTouched)
		}
	}
	return 1
}

do_victim(victim,attacker,damage,unarmed,tk){
	new namek[32],namev[32],authida[40],authidv[40],teama[32],teamv[32]
	get_user_name(victim,namev,31)
	get_user_name(attacker,namek,31)
	get_user_authid(victim,authidv,39)
	get_user_authid(attacker,authida,39)
	get_user_team(victim,teamv,31)
	get_user_team(attacker,teama,31)

	if(unarmed == 1){
		if(damage > 99)
			client_print(attacker,print_chat,"[AMX] NON-LETHAL TEST MODE:  You would have killed %s with that missile",namev)		
		else
			client_print(attacker,print_chat,"[AMX] NON-LETHAL TEST MODE:  You would have hurt %s with that missile",namev)	
	}else{
		if(damage > 99){
			user_kill(victim,1)
			if(is_heat_rocket[attacker] == 1)
				set_task(1.0,"delay_arnold")
			if(unarmed == 2){
				log_to_file("addons/amx/admin.log","^"%s<%d><%s><%s>^" admin missile killed ^"%s<%d><%s><%s>^"",
					namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv)
				client_print(attacker,print_chat,"[AMX] You killed %s with that admin missile",namev)
				client_print(victim,print_chat,"[AMX] You were killed by %s's admin missile",namek)
				#if use_newdeathmsg == 1
				stop_dm(victim,victim,1,9876)
				#endif
				server_cmd("artificial_dm %d %d",attacker,victim)		
			}else{
				if(get_cvar_num("mp_logdetail") == 3){
					log_message("^"%s<%d><%s><%s>^" attacked ^"%s<%d><%s><%s>^" with ^"Missile^" (hit ^"head^") (damage ^"%d^") (health ^"0^")",
					namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv,damage)
				}
				log_message("^"%s<%d><%s><%s>^" killed ^"%s<%d><%s><%s>^" with ^"Missile^"",
					namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv)
				client_print(attacker,print_chat,"[AMX] You killed %s with that missile",namev)
				client_print(victim,print_chat,"[AMX] You were killed by %s's missile",namek)		
				if(tk == 0){
					set_user_frags(attacker,get_user_frags(attacker) +1 )
				}else{
					tkcount[attacker] +=1
					client_print(attacker,print_center,"You killed a teammate")
					set_user_frags(attacker,get_user_frags(attacker) -1 )
				}
				#if use_newdeathmsg == 1
				stop_dm(attacker,victim,1,9876)
				#endif
				server_cmd("artificial_dm %d %d",attacker,victim)	
			}
		}else{
			set_user_health(victim,get_user_health(victim) - damage )
			if(unarmed == 2){
				log_to_file("addons/amx/admin.log","^"%s<%d><%s><%s>^" admin missile hurt ^"%s<%d><%s><%s>^"",
					namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv)
				if(get_user_health(victim) < 1){
					if(is_heat_rocket[attacker] == 1)
						set_task(1.0,"delay_arnold")
					log_to_file("addons/amx/admin.log","^"%s<%d><%s><%s>^" admin missile killed ^"%s<%d><%s><%s>^"",
						namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv)
					client_print(attacker,print_chat,"[AMX] You killed %s with that admin missile",namev)
					client_print(victim,print_chat,"[AMX] You were killed by %s's admin missile",namek)
					#if use_newdeathmsg == 1
					stop_dm(victim,victim,0,9876)
					#endif
					server_cmd("artificial_dm %d %d",attacker,victim)	
				}else{
					client_print(attacker,print_chat,"[AMX] You hurt %s with that admin missile",namev)
					client_print(victim,print_chat,"[AMX] You were hurt by %s's admin missile",namek)
				}
			}else{			
				if(get_cvar_num("mp_logdetail") == 3){
					log_message("^"%s<%d><%s><%s>^" attacked ^"%s<%d><%s><%s>^" with ^"Missile^" (hit ^"chest^") (damage ^"%d^") (health ^"%d^")",
					namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv,damage,get_user_health(victim))
				}
				if(get_user_health(victim) < 1){
					if(is_heat_rocket[attacker] == 1)
						set_task(1.0,"delay_arnold")
					log_message("^"%s<%d><%s><%s>^" killed ^"%s<%d><%s><%s>^" with ^"Missile^"",
						namek,get_user_userid(attacker),authida,teama,namev,get_user_userid(victim),authidv,teamv)
					client_print(attacker,print_chat,"[AMX] You killed %s with that missile",namev)
					client_print(victim,print_chat,"[AMX] You were killed by %s's missile",namek)
					if(tk == 0){	
						set_user_frags(attacker,get_user_frags(attacker) +1 )
					}else{
						tkcount[attacker] +=1
						client_print(attacker,print_center,"You killed a teammate")
						set_user_frags(attacker,get_user_frags(attacker) -1 )
					}
					set_user_frags(victim,get_user_frags(victim) +1 )
					#if use_newdeathmsg == 1
					stop_dm(attacker,victim,0,9876)
					#endif
					server_cmd("artificial_dm %d %d",attacker,victim)	
				}else{
					client_print(attacker,print_chat,"[AMX] You hurt %s with that missile",namev)
					client_print(victim,print_chat,"[AMX] You were hurt by %s's missile",namek)
				}
			}		
		}
		if(tk == 1 && unarmed == 0){
			new players[32],pNum
			get_players(players,pNum,"e",teama)
			for(new i=0;i<pNum;i++)
				client_print(players[i],print_chat,"%s  attacked a teammate",namek)
			new punish1 = get_cvar_num("amx_missile_tkpunish1")
			new punish2 = get_cvar_num("amx_missile_tkpunish2")
			if(punish1 > 2){
				user_kill(attacker,0)
				set_hudmessage(255,50,50, -1.0, 0.45, 0, 0.02, 10.0, 1.01, 1.1, 4)	
				show_hudmessage(attacker,"YOU WERE KILLED^nFOR ATTACKING TEAMMATES.^nSEE THAT IT HAPPENS NO MORE!") 
			}
			if((punish1) && (tkcount[attacker] > punish2 - 1)){
				if(punish1 == 1 || punish1 == 3) 
					client_cmd(attacker,"echo You were kicked for team killing;disconnect")
				else if(punish1 == 2 || punish1 == 4){
					client_cmd(attacker,"echo You were banned for team killing")
					if (equal("4294967295",authida)){
						new ipa[32]
						get_user_ip(attacker,ipa,31,1)
						server_cmd("addip 180.0 %s;writeip",ipa)
					}else{
						server_cmd("banid 180.0 %s kick;writeid",authida)
					}	
				}
			}
		}
	}
}

public delay_arnold(){
	client_cmd(0,"play misc/arnold_heatseeker.wav")
	return PLUGIN_CONTINUE
}

public player_sdmf(id){
	if(get_cvar_num("amx_missile_allow_sdmf") == 0){
		console_print(id,"[AMX] Admin is not allowing players to set thier own fuel for swirling death missiles.")
		return PLUGIN_HANDLED
	}
	if (read_argc() < 2){
		console_print(id,"[AMX] Usage: amx_sdmf <fuel ammount in 10ths/second>. To make 1 second:  amx_sdmf 10")	
		return PLUGIN_HANDLED
	}
	new arg[8]
	read_argv(1,arg,7)
	new iarg = strtonum(arg)	
	if(iarg < 1 || iarg > 1000){
		console_print(id,"[AMX] Bad data. Fuel ammount must be a number between 1 and 1000")	
		return PLUGIN_HANDLED
	}
	user_sdmf[id] = (iarg + 0.0) / 10  	
	console_print(id,"[AMX] Your swirling death missiles now run out of fuel after %.1f seconds",user_sdmf[id])		
	return PLUGIN_HANDLED
}

public admin_missile(id,level,cid){ 
	if (!cmd_access(id,level,cid,1)) 
		return PLUGIN_HANDLED
	if(!is_dedicated_server()){
		if(!id)
			id = 1
	}
	if( (has_rocket[id]) && (!(get_user_flags(id)&ADMIN_LEVEL_D)) ){
		client_print(id,print_chat,"[AMX] You cannot have more than one missile in the air at a time.")
		return PLUGIN_HANDLED
	}
	using_menu[id] = 0	
	new arg1[10]
	new arg2[10]
	new arg3[10]
	new cmd[32],icmd
	read_argv(0,cmd,31)
	read_argv(1,arg1,9)
	new iarg1 = strtonum(arg1)
	read_argv(2,arg2,9)
	new iarg2 = strtonum(arg2)
	read_argv(3,arg3,9)
	new iarg3 = strtonum(arg3)
	if(equal(cmd[11],"1",1))
		icmd = 1
	else if(equal(cmd[11],"2",1))
		icmd = 2
	else if(equal(cmd[11],"3",1))
		icmd = 3
	else if(equal(cmd[11],"4",1))
		icmd = 4
	else
		icmd = 7
	if(iarg1 < 1 || iarg1 > 10000){
		if(icmd == 7)
			iarg1 = get_cvar_num("amx_missile_sdspeed")
		else if(icmd == 3)
			iarg1 = get_cvar_num("amx_missile_hsspeed")	
		else
			iarg1 = get_cvar_num("amx_missile_speed")
	}
	if(iarg2 == 1 && !cstrike_running)
		iarg2 = 0
	if(iarg2 > 1)
		iarg2 = 0  
	make_rocket(id,icmd,iarg1,iarg2,iarg3,1,0)
	return PLUGIN_HANDLED
}		

public bot_interface(){
	if(get_cvar_num("amx_luds_missiles") == 0)
		return PLUGIN_HANDLED
	new sid[8],id
	read_argv(1,sid,7)
	id = strtonum(sid)
	if(is_user_alive(id) == 0)
		return PLUGIN_HANDLED
	if(has_rocket[id]) 
		return PLUGIN_HANDLED
	new cmd[32],icmd
	read_argv(2,cmd,31)
	if(equal(cmd,"amx_guncamera_missile"))
		icmd = 1		
	else if(equal(cmd,"amx_laserguided_missile"))
		icmd = 2
	else if(equal(cmd,"amx_heatseeking_missile"))
		icmd = 3
	else if(equal(cmd,"amx_missile"))
		icmd = 4
	else if(equal(cmd,"amx_ropeseeking_missile"))
		icmd = 6
	else if(equal(cmd,"amx_swirlingdeath_missile"))
		icmd = 7
	else 
		icmd = 5
	if(icmd == 5){
		if(radar_batt[id] < 1)
			return PLUGIN_HANDLED
		if(is_scan_rocket[id] == 1) {
			remove_task(35632+id)
			is_scan_rocket[id] = 0
			missile_inv[id][icmd] += 1
			return PLUGIN_HANDLED
		}
	}
	if( (icmd == 1 || icmd == 7) && (round_st == 1) && (get_cvar_num("amx_missile_atspawn") == 0) )
		return PLUGIN_HANDLED
	if(get_cvar_num("amx_missile_botsnolimit") == 0){ 
		if(get_cvar_num("amx_missile_buy") == 1){
			new cvarname[32]
			format(cvarname,31,"amx_missile_cost%d",icmd)
			new umoney = get_user_money(id)
			new m_cost = get_cvar_num(cvarname)
			if(umoney < m_cost)
				return PLUGIN_HANDLED
			else
				set_user_money(id,umoney-m_cost,1)		
		}else{
			new pass
			if( (icmd == 7) && (get_cvar_num("amx_missile_ammo7takeall") == 1) ){
				pass = 1
				new sum
				for(new i=1;i<8;i++)
					sum += missile_inv[id][i]
				if(sum < 7)				
					return PLUGIN_HANDLED
				else{
					new take
					for(new b=1;b<8;b++){
						if(take < 7){
							for(new i=1;i<8;i++){
								if( (missile_inv[id][i] > 0) && (take < 7) ){ 				
									missile_inv[id][i] -= 1
									take +=1
								}
							}
						}
					}
				}		
			}
			if( (!pass) && (missile_inv[id][icmd] < 1) )
				return PLUGIN_HANDLED
			else
				missile_inv[id][icmd] -= 1
		}
	}
	if(icmd != 5){
		switch(icmd){
			case 1: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 2: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 3: make_rocket(id,icmd,get_cvar_num("amx_missile_hsspeed"),0,1,0,0)
			case 4: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 6: make_rocket(id,icmd,get_cvar_num("amx_missile_rsspeed"),0,1,0,0)
			case 7: make_rocket(id,icmd,get_cvar_num("amx_missile_sdspeed"),0,1,0,0)
			default: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
		}
	}else{
		is_scan_rocket[id] = 1
		new args[2]
		args[0] = id
		set_task(0.2,"anti_missile_radar",35632+id,args,2,"b")
		set_task(0.3,"amr_pay",35632+id,args,2,"b")
	}
	return PLUGIN_CONTINUE
}

public fire_missile(id){
	if(!is_dedicated_server()){
		if(!id)
			id = 1
	}
	if(get_cvar_num("amx_luds_missiles") == 0){
		client_print(id,print_chat,"[AMX] Missiles are disabled right now")
		return PLUGIN_HANDLED
	}
	if(is_user_alive(id) == 0){
		client_print(id,print_chat,"[AMX] You cannot use a missile when you are dead.")
		return PLUGIN_HANDLED
	}
	show_missile_inv(id)
	if(has_rocket[id]){ 
		client_print(id,print_chat,"[AMX] You cannot have more than one missile in the air at a time.")
		return PLUGIN_HANDLED
	}

	new cmd[32],icmd
	read_argv(0,cmd,31)
	if(equal(cmd,"amx_guncamera_missile"))
		icmd = 1		
	else if(equal(cmd,"amx_laserguided_missile"))
		icmd = 2
	else if(equal(cmd,"amx_heatseeking_missile"))
		icmd = 3
	else if(equal(cmd,"amx_missile"))
		icmd = 4
	else if(equal(cmd,"amx_ropeseeking_missile"))
		icmd = 6
	else if(equal(cmd,"amx_swirlingdeath_missile"))
		icmd = 7
	else 
		icmd = 5
	if(icmd == 5){
		if(radar_batt[id] < 1){
			client_print(id,print_chat,"[AMX] Your anti-missile radar batteries are dead.")
			return PLUGIN_HANDLED
		}
		if(is_scan_rocket[id] == 1) {
			remove_task(35632+id)
			is_scan_rocket[id] = 0
			missile_inv[id][icmd] += 1
			set_hudmessage(0,255,0, -1.0, 0.30, 0, 0.02, 3.0, 1.01, 1.1, 4)	
			show_hudmessage(id,"ANTIMISSILE RADAR DEACTIVATED")
			return PLUGIN_HANDLED
		}
	} 
	if( (icmd == 1 || icmd == 7) && (round_st == 1) && (get_cvar_num("amx_missile_atspawn") == 0) ){
		client_print(id,print_chat,"[AMX] This missile type cannot be fired until 15 seconds after round start.")
		return PLUGIN_HANDLED
	}
	if(get_cvar_num("amx_missile_buy") == 1){
		new cvarname[32]
		format(cvarname,31,"amx_missile_cost%d",icmd)
		new umoney = get_user_money(id)
		new m_cost = get_cvar_num(cvarname)
		if(umoney < m_cost){
			client_print(id,print_chat,"[AMX]  Insufficient funds.  Each of these missiles costs %d money",m_cost)
			return PLUGIN_HANDLED
		}else{
			set_user_money(id,umoney-m_cost,1)
		}
	}else{
		new pass
		if( (icmd == 7) && (get_cvar_num("amx_missile_ammo7takeall") == 1) ){
			pass = 1
			new sum
			for(new i=1;i<8;i++)
				sum += missile_inv[id][i]
			if(sum < 7){				
				client_print(id,print_chat,"[AMX] You do not have enough missiles to make a swirling death missile.")
				return PLUGIN_HANDLED
			}else{
				new take
				for(new b=1;b<8;b++){
					if(take < 7){
						for(new i=1;i<8;i++){
							if( (missile_inv[id][i] > 0) && (take < 7) ){ 				
								missile_inv[id][i] -= 1
								take +=1
							}
						}
					}
				}
			}		
		}
		if( (!pass) && (missile_inv[id][icmd] < 1) ){
			switch(icmd){
				case 1: client_print(id,print_chat,"[AMX] You have no more guncamera missiles.")
				case 2: client_print(id,print_chat,"[AMX] You have no more laserguided missiles.")
				case 3: client_print(id,print_chat,"[AMX] You have no more heatseeking missiles.")
				case 4: client_print(id,print_chat,"[AMX] You have no more stupid missiles.")
				case 5: client_print(id,print_chat,"[AMX] You have no more anti-missile missiles.")
				case 6: client_print(id,print_chat,"[AMX] You have no more ropeseeking missiles.")
				case 7: client_print(id,print_chat,"[AMX] You have no more swirling death missiles.")
			}
			return PLUGIN_HANDLED
		}else{
			missile_inv[id][icmd] -= 1
		}
	}
	if(icmd != 5){
		show_missile_inv(id)
		switch(icmd){
			case 1: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 2: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 3: make_rocket(id,icmd,get_cvar_num("amx_missile_hsspeed"),0,1,0,0)
			case 4: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
			case 6: make_rocket(id,icmd,get_cvar_num("amx_missile_rsspeed"),0,1,0,0)
			case 7: make_rocket(id,icmd,get_cvar_num("amx_missile_sdspeed"),0,1,0,0)
			default: make_rocket(id,icmd,get_cvar_num("amx_missile_speed"),0,1,0,0)
		}
	}else{
		is_scan_rocket[id] = 1
		new args[2]
		args[0] = id
		set_task(0.2,"anti_missile_radar",35632+id,args,2,"b")
		set_task(0.3,"amr_pay",35632+id,args,2,"b")
		set_hudmessage(0,255,0, -1.0, 0.30, 0, 0.02, 3.0, 1.01, 1.1, 4)	
		show_hudmessage(id,"ANTIMISSILE:^nRADAR SYSTEM ACTIVATED^n(aim at missile you want to shoot down)")
	}
	return PLUGIN_HANDLED
}

public anti_missile_radar(args[]){
	new maxplayers = get_maxplayers()+1
	new id = args[0]
	new tid = 0
	new aimvec[3],origin[3],length
	new radarvec1[3],radarvec2[3],radarvec3[3],radarvec4[3],radarvec5[3]
	get_user_origin(id,origin)
	get_user_origin(id,aimvec,3)
	radarvec1[0]=aimvec[0]-origin[0]
	radarvec1[1]=aimvec[1]-origin[1]
	radarvec1[2]=aimvec[2]-origin[2] 
	length=sqrt(radarvec1[0]*radarvec1[0]+radarvec1[1]*radarvec1[1]+radarvec1[2]*radarvec1[2]) 
	radarvec5[0]=radarvec1[0]*1750/length + origin[0]
	radarvec5[1]=radarvec1[1]*1750/length + origin[1]
	radarvec5[2]=radarvec1[2]*1750/length + origin[2]
	radarvec4[0]=radarvec1[0]*1350/length + origin[0]
	radarvec4[1]=radarvec1[1]*1350/length + origin[1]
	radarvec4[2]=radarvec1[2]*1350/length + origin[2]
	radarvec3[0]=radarvec1[0]*950/length + origin[0]
	radarvec3[1]=radarvec1[1]*950/length + origin[1]
	radarvec3[2]=radarvec1[2]*950/length + origin[2]
	radarvec2[0]=radarvec1[0]*700/length + origin[0]
	radarvec2[1]=radarvec1[1]*700/length + origin[1]
	radarvec2[2]=radarvec1[2]*700/length + origin[2]
	radarvec1[0]=radarvec1[0]*350/length + origin[0]
	radarvec1[1]=radarvec1[1]*350/length + origin[1]
	radarvec1[2]=radarvec1[2]*350/length + origin[2]

	for(new i=1; i<=maxplayers; i++) {
		if( (has_rocket[i] > maxplayers) && (i != id) && (tid < maxplayers) ){
			new szClassName[32]
			Entvars_Get_String(has_rocket[i], EV_SZ_classname, szClassName, 31)
			if(equal(szClassName, "lud_missile")){
				new rocketvec[3]
				new Float:fl_rocketvec[3]
				Entvars_Get_Vector(has_rocket[i], EV_VEC_origin, fl_rocketvec)
				rocketvec[0] = floatround(fl_rocketvec[0])
				rocketvec[1] = floatround(fl_rocketvec[1])
				rocketvec[2] = floatround(fl_rocketvec[2])
				if(get_distance(radarvec5,rocketvec) < 100)
					tid = has_rocket[i]
				else if(get_distance(radarvec4,rocketvec) < 100)
					tid = has_rocket[i]
				else if(get_distance(radarvec3,rocketvec) < 85)
					tid = has_rocket[i]
				else if(get_distance(radarvec2,rocketvec) < 70)
					tid = has_rocket[i]
				else if(get_distance(radarvec1,rocketvec) < 50)
					tid = has_rocket[i]
			}
		}
	}
	if(tid > maxplayers){
		client_cmd(id,"spk fvox/beep")
		set_hudmessage(255,0,0, -1.0, 0.30, 0, 0.02, 3.0, 1.01, 1.1, 4)	
		show_hudmessage(id,"ANTIMISSILE LOCKED ONTO TARGET")
		is_scan_rocket[id] = 0
		remove_task(35632+id)
		make_rocket(id,5,get_cvar_num("amx_missile_speed")*5,0,1,0,tid)
	}
	return PLUGIN_CONTINUE
}

public amr_pay(args[]){
	new id = args[0]
	set_hudmessage(255,0,0, -1.0, 0.30, 0, 0.02, 3.0, 1.01, 1.1, 4)
	if(radar_batt[id] < 1){
		show_hudmessage(id,"WARNING: ANTIMISSILE RADAR SYSTEM FAILURE")
		client_print(id,print_center,"^n^nBattery is dead")	
		is_scan_rocket[id] = 0
		remove_task(35632+id)
	}else{
		client_print(id,print_center,"^n^n^nBattery %d",radar_batt[id])
	}
	radar_batt[id] -= 1
	client_cmd(id,"spk fvox/blip")
	return PLUGIN_CONTINUE
}

make_rocket(id,icmd,iarg1,iarg2,iarg3,admin,antimissile){ 
	/* userindex,commandtype,missilespeed,model,nofake,admincommand,antimissleid */
	new args[16]
	new Float:vOrigin[3]
	new Float:vAngles[3]
	Entvars_Get_Vector(id, EV_VEC_origin, vOrigin)
	Entvars_Get_Vector(id, EV_VEC_v_angle, vAngles)
	new notFloat_vOrigin[3]
	notFloat_vOrigin[0] = floatround(vOrigin[0])
	notFloat_vOrigin[1] = floatround(vOrigin[1])
	notFloat_vOrigin[2] = floatround(vOrigin[2])

	if(icmd == 3){
		new aimvec[3]
		get_user_origin(id,aimvec,3)
		new dist = get_distance(notFloat_vOrigin,aimvec)
		new found
		new dist1 = 20000

		new players[32], inum
		get_players(players,inum,"a")
		for(new i = 0 ;i < inum ;++i){
			if(players[i] != id){
				new soutput[8],output 
				get_user_info(players[i],"JETPACK_RUN",soutput,7)
				output = strtonum(soutput)
				if(cmd_jetpackrun[players[i]] == 1)
					output = 1
				if(output){
					new temp[3]
					get_user_origin(players[i],temp)
					dist1 = get_distance(temp,aimvec)
					if(dist1 < dist){
						dist = dist1
						found = 1
						args[6] = players[i]
					}
				}
			}
		}
		if(!found){
			client_print(id,print_chat,"[AMX]  Cannot fire heat seeking missile, no running jetpacks in view.")
			if(!admin){
				if(get_cvar_num("amx_missile_buy") == 1){
					new umoney = get_user_money(id)
					new m_cost = get_cvar_num("amx_missile_cost3")
					set_user_money(id,umoney+m_cost,1)
				}else{
					missile_inv[id][3] += 1
					show_missile_inv(id)
				}
				if(using_menu[id] == 1)
					show_main_menu(id)
			}
			return PLUGIN_HANDLED			
		}
	}
	else if(icmd == 6){
		new aimvec[3]
		get_user_origin(id,aimvec,3)
		new dist = get_distance(notFloat_vOrigin,aimvec)
		new found
		new dist1 = 20000
		new players[32], inum
		get_players(players,inum,"a")
		for(new i = 0 ;i < inum ;++i){
			if(players[i] != id){
				if(cmd_onrope[players[i]] == 1){
					new temp[3]
					get_user_origin(players[i],temp)
					dist1 = get_distance(temp,aimvec)
					if(dist1 < dist){
						dist = dist1
						found = 1
						args[6] = players[i]
					}
				}
			}
		}
		if(!found){
			client_print(id,print_chat,"[AMX]  Cannot fire rope seeking missile, no ropes in view.")
			if(!admin){
				if(get_cvar_num("amx_missile_buy") == 1){
					new umoney = get_user_money(id)
					new m_cost = get_cvar_num("amx_missile_cost6")
					set_user_money(id,umoney+m_cost,1)
				}else{
					missile_inv[id][6] += 1
					show_missile_inv(id)
				}
				if(using_menu[id] == 1)
					show_main_menu(id)
			}
			return PLUGIN_HANDLED			
		}
	}
	using_menu[id] = 0

	new NewEnt
	NewEnt = CreateEntity("info_target")
	if(NewEnt == 0) {
		client_print(id,print_chat,"Rocket failure")
		return PLUGIN_HANDLED_MAIN
	}
	has_rocket[id] = NewEnt
	if(admin == 1){
		if(iarg3 == 1)
			missile_inv[id][0] = 2
		else
			missile_inv[id][0] = 1
	}else{
		missile_inv[id][0] = 0
	}
	Entvars_Set_String(NewEnt, EV_SZ_classname, "lud_missile")
	switch(iarg2){
		case 0: ENT_SetModel(NewEnt, "models/rpgrocket.mdl")
		case 1: ENT_SetModel(NewEnt, "models/chick.mdl")
		// case 2: ENT_SetModel(NewEnt, "models/hvr.mdl")
	}

	new Float:fl_vecmins[3] = {-1.0, -1.0, -1.0}
	new Float:fl_vecmaxs[3] = {1.0, 1.0, 1.0}

	Entvars_Set_Vector(NewEnt, EV_VEC_mins,fl_vecmins)
	Entvars_Set_Vector(NewEnt, EV_VEC_maxs,fl_vecmaxs)

	ENT_SetOrigin(NewEnt, vOrigin)
	Entvars_Set_Vector(NewEnt, EV_VEC_angles, vAngles)

	if(!iarg2)
		Entvars_Set_Int(NewEnt, EV_INT_effects, 64)  
	else
		Entvars_Set_Int(NewEnt, EV_INT_effects, 2)  
	Entvars_Set_Int(NewEnt, EV_INT_solid, 2)    
	if(get_cvar_num("amx_missile_obeygravity"))
		Entvars_Set_Int(NewEnt, EV_INT_movetype, 6) 
	else
		Entvars_Set_Int(NewEnt, EV_INT_movetype, 5) 
	Entvars_Set_Edict(NewEnt, EV_ENT_owner, id)
	Entvars_Set_Float(NewEnt, EV_FL_health, 10000.0)
	Entvars_Set_Float(NewEnt, EV_FL_takedamage, 100.0)
	Entvars_Set_Float(NewEnt, EV_FL_dmg_take, 100.0)
	
	new Float:fl_iNewVelocity[3]
	new iNewVelocity[3]
	VelocityByAim(id, iarg1, fl_iNewVelocity)
	Entvars_Set_Vector(NewEnt, EV_VEC_velocity, fl_iNewVelocity)
	iNewVelocity[0] = floatround(fl_iNewVelocity[0]) 
	iNewVelocity[1] = floatround(fl_iNewVelocity[1]) 
	iNewVelocity[2] = floatround(fl_iNewVelocity[2]) 

	emit_sound(NewEnt, CHAN_WEAPON, "weapons/rocketfire1.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
	emit_sound(NewEnt, CHAN_VOICE, "weapons/rocket1.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)

	args[0] = id
	args[1] = NewEnt
	args[2] = iarg1
	args[3] = iNewVelocity[0]
	args[4] = iNewVelocity[1]
	args[5] = iNewVelocity[2]
	args[8] = notFloat_vOrigin[0]
	args[9] = notFloat_vOrigin[1]
	args[10] = notFloat_vOrigin[2]

	switch(icmd){
		case 1: {
			make_trail(NewEnt,icmd)
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			Entvars_Set_Int(NewEnt, EV_INT_rendermode,1) 
			AttachView(id, NewEnt)
			args[11] = 1
			set_task(0.1,"guide_rocket_dir",2020+NewEnt,args,16,"b")
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 2: {
			make_trail(NewEnt,icmd) 
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			set_task(0.1,"guide_rocket_las",2020+NewEnt,args,16)
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 3: {
			is_heat_rocket[id] = 1
			make_trail(NewEnt,icmd)
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			set_task(0.1,"guide_rocket_het",2020+NewEnt,args,16)
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 4: {
			make_trail(NewEnt,icmd)
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			set_task(0.1,"guide_rocket_comm",2020+NewEnt,args,16,"b")
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 5: {
			make_trail(NewEnt,icmd)
			args[6] = antimissile
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			set_task(0.1,"guide_rocket_anti",2020+NewEnt,args,16)
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 6: {
			make_trail(NewEnt,icmd)
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.25)
			set_task(0.1,"guide_rocket_rope",2020+NewEnt,args,16)
			set_task(get_cvar_float("amx_missile_fuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
		case 7: {
			Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.000001)
			SD_CircleRockets(NewEnt) 
			set_task(0.1,"guide_rocket_swirl",2020+NewEnt,args,16)
			if( (user_sdmf[id] > 0.0) && (get_cvar_num("amx_missile_allow_sdmf") == 1) )
				set_task(user_sdmf[id],"rocket_fuel_timer",2020+NewEnt,args,16)
			else
				set_task(get_cvar_float("amx_missile_sdfuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
		}
	}
	return PLUGIN_HANDLED_MAIN
}

make_trail(NewEnt,style){
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY) 
	write_byte(22) 
	write_short(NewEnt) 
	write_short(beam) 
	write_byte(45) 
	write_byte(4)
	switch(style){
		case 3: { 
			write_byte(250) 
			write_byte(100) 
			write_byte(100)
			write_byte(100) 
		}
		case 6: {
			write_byte(100) 
			write_byte(250) 
			write_byte(100)
			write_byte(100) 
		}
		case 7: {
			write_byte(100) 
			write_byte(100) 
			write_byte(250)
			write_byte(100) 
		}

		default: {
			write_byte(250) 
			write_byte(250) 
			write_byte(250)
			write_byte(100)
		}
	} 
	message_end() 
}

public guide_rocket_comm(args[]){
	new ent = args[1]
	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)

	return PLUGIN_CONTINUE
}

public guide_rocket_dir(args[]){
	new id = args[0]
	new ent = args[1]
	new speed = args[2]
	new Float:fl_iNewVelocity[3] 
	VelocityByAim(id, speed, fl_iNewVelocity)
	Entvars_Set_Vector(ent, EV_VEC_velocity, fl_iNewVelocity)

	new Float:vAngles[3]
	Entvars_Get_Vector(id, EV_VEC_v_angle, vAngles)
	Entvars_Set_Vector(ent, EV_VEC_angles, vAngles)

	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)		
	return PLUGIN_CONTINUE
}

public guide_rocket_las(args[]){
	new aimvec[3],avgFactor
	new Float:fl_origin[3]
	new id = args[0]
	new ent = args[1]
	new speed = args[2]
	get_user_origin(id,aimvec,3)
	make_dot(aimvec)  
	Entvars_Get_Vector(ent, EV_VEC_origin, fl_origin)
	new iNewVelocity[3]
	new origin[3]
	origin[0] = floatround(fl_origin[0])
	origin[1] = floatround(fl_origin[1])
	origin[2] = floatround(fl_origin[2])
	if(speed < 400)
		avgFactor = 10
	else if(speed < 850)
		avgFactor = 4
	else
		avgFactor = 2
	new velocityvec[3],length
	velocityvec[0]=aimvec[0]-origin[0] 
	velocityvec[1]=aimvec[1]-origin[1]
	velocityvec[2]=aimvec[2]-origin[2]
	length=sqrt(velocityvec[0]*velocityvec[0]+velocityvec[1]*velocityvec[1]+velocityvec[2]*velocityvec[2])
	velocityvec[0]=velocityvec[0]*speed/length 
	velocityvec[1]=velocityvec[1]*speed/length 
	velocityvec[2]=velocityvec[2]*speed/length 
	iNewVelocity[0] = (velocityvec[0] + (args[3] * (avgFactor-1) ) ) / avgFactor
	iNewVelocity[1] = (velocityvec[1] + (args[4] * (avgFactor-1) ) ) / avgFactor
	iNewVelocity[2] = (velocityvec[2] + (args[5] * (avgFactor-1) ) ) / avgFactor
	new Float:fl_iNewVelocity[3] 
	fl_iNewVelocity[0] = iNewVelocity[0] + 0.0
	fl_iNewVelocity[1] = iNewVelocity[1] + 0.0
	fl_iNewVelocity[2] = iNewVelocity[2] + 0.0
	Entvars_Set_Vector(ent, EV_VEC_velocity, fl_iNewVelocity)
	args[3] = iNewVelocity[0]
	args[4] = iNewVelocity[1]
	args[5] = iNewVelocity[2]
	args[8] = origin[0]
	args[9] = origin[1]
	args[10] = origin[2]
	set_task(0.1,"guide_rocket_las",2020+ent,args,16)

	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)	

	return PLUGIN_CONTINUE
}

public guide_rocket_het(args[]){
	new aimvec[3],avgFactor
	new Float:fl_origin[3]
	new t_aimvec[33][3]
	new t_index[33]
	new t_jp,dist
	new jp_dist = 20000
	new id = args[0]
	new ent = args[1]
	new speed = args[2]
	new iNewVelocity[3]
	Entvars_Get_Vector(ent, EV_VEC_origin, fl_origin)
	new origin[3]
	origin[0] = floatround(fl_origin[0])
	origin[1] = floatround(fl_origin[1])
	origin[2] = floatround(fl_origin[2])

	new soutput[8],output 
	get_user_info(args[6],"JETPACK_RUN",soutput,7)
	output = strtonum(soutput)
	if(cmd_jetpackrun[args[6]] == 1)
		output = 1

	if((output == 1) || (args[7] < 5)){
		if(is_user_alive(args[6]) == 1){
			get_user_origin(args[6],aimvec)
			dist = get_distance(aimvec,origin)
		}else{
			args[7] = 100
		}
		if(output == 1)
			args[7] = 0
	}else{
		new players[32], inum
		get_players(players,inum,"a")
		for(new i = 0 ;i < inum ;++i){
			if(players[i] != id){
				setc(soutput,8,0)	
				get_user_info(players[i],"JETPACK_RUN",soutput,7)
				output = strtonum(soutput)
				if(cmd_jetpackrun[players[i]] == 1)
					output = 1
				if(output){
					new temp[3]
					get_user_origin(players[i],temp)
					t_aimvec[t_jp][0] = temp[0]
					t_aimvec[t_jp][1] = temp[1]
					t_aimvec[t_jp][2] = temp[2]
					t_index[t_jp] = players[i]
					t_jp++
				}
			}
		}
		for(new i = 0 ;i < t_jp ;++i){
			new temp[3]
			temp[0] = t_aimvec[i][0]
			temp[1] = t_aimvec[i][1]
			temp[2] = t_aimvec[i][2]
			dist = get_distance(temp,origin)
			if(dist < jp_dist){
				aimvec[0] = temp[0]
				aimvec[1] = temp[1]
				aimvec[2] = temp[2]
				jp_dist = dist
				args[6] = t_index[i]
				args[7] = 0	
			}
		}	
	}	  
	if(dist){
		if(speed < 400)
			avgFactor = 10
		else if(speed < 850)
			avgFactor = 4
		else
			avgFactor = 2

		new length, velocityvec[3]
		velocityvec[0]=aimvec[0]-origin[0] 
		velocityvec[1]=aimvec[1]-origin[1] 
		velocityvec[2]=aimvec[2]-origin[2] 
		length=sqrt(velocityvec[0]*velocityvec[0]+velocityvec[1]*velocityvec[1]+velocityvec[2]*velocityvec[2]) 
		velocityvec[0]=velocityvec[0]*speed/length 
		velocityvec[1]=velocityvec[1]*speed/length 
		velocityvec[2]=velocityvec[2]*speed/length 

		iNewVelocity[0] = (velocityvec[0] + (args[3] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[1] = (velocityvec[1] + (args[4] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[2] = (velocityvec[2] + (args[5] * (avgFactor-1) ) ) / avgFactor

		args[3] = iNewVelocity[0]
		args[4] = iNewVelocity[1]
		args[5] = iNewVelocity[2]
		if(dist < 20){
			vexd_pfntouch(ent,0)
			return PLUGIN_CONTINUE
		}			
	}
	args[7] += 1
	new Float:fl_iNewVelocity[3]
	fl_iNewVelocity[0] = args[3] +0.0
	fl_iNewVelocity[1] = args[4] +0.0
	fl_iNewVelocity[2] = args[5] +0.0

	Entvars_Set_Vector(ent, EV_VEC_velocity, fl_iNewVelocity)
	set_task(0.1,"guide_rocket_het",2020+ent,args,16)

	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)	

	return PLUGIN_CONTINUE
}

public guide_rocket_anti(args[]){
	new Float:fl_aimvec[3]
	new Float:fl_origin[3]
	new avgFactor
	new id = args[0]
	new ent = args[1]
	new speed = args[2]
	new iNewVelocity[3]

	Entvars_Get_Vector(ent, EV_VEC_origin, fl_origin)
	new origin[3]
	origin[0] = floatround(fl_origin[0])
	origin[1] = floatround(fl_origin[1])
	origin[2] = floatround(fl_origin[2])

	if(FindEntity(args[6], "lud_missile") != -1){	
		Entvars_Get_Vector(args[6], EV_VEC_origin, fl_aimvec)
	  	new aimvec[3]
		aimvec[0] = floatround(fl_aimvec[0])
		aimvec[1] = floatround(fl_aimvec[1])
		aimvec[2] = floatround(fl_aimvec[2])
		if(speed < 400)
			avgFactor = 10
		else if(speed < 850)
			avgFactor = 4
		else
			avgFactor = 2
		new length, velocityvec[3]
		velocityvec[0]=aimvec[0]-origin[0] 
		velocityvec[1]=aimvec[1]-origin[1] 
		velocityvec[2]=aimvec[2]-origin[2] 
		length=sqrt(velocityvec[0]*velocityvec[0]+velocityvec[1]*velocityvec[1]+velocityvec[2]*velocityvec[2]) 
		velocityvec[0]=velocityvec[0]*speed/length 
		velocityvec[1]=velocityvec[1]*speed/length 
		velocityvec[2]=velocityvec[2]*speed/length 
		iNewVelocity[0] = (velocityvec[0] + (args[3] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[1] = (velocityvec[1] + (args[4] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[2] = (velocityvec[2] + (args[5] * (avgFactor-1) ) ) / avgFactor
		args[3] = iNewVelocity[0]
		args[4] = iNewVelocity[1]
		args[5] = iNewVelocity[2]
		if(get_distance(origin,aimvec) < 150){
			vexd_pfntouch(ent,args[6])
			return PLUGIN_CONTINUE
		}			
		new Float:fl_iNewVelocity[3]
		fl_iNewVelocity[0] = args[3] +0.0
		fl_iNewVelocity[1] = args[4] +0.0
		fl_iNewVelocity[2] = args[5] +0.0
		Entvars_Set_Vector(ent, EV_VEC_velocity, fl_iNewVelocity)
		set_task(0.1,"guide_rocket_anti",2020+ent,args,16)
	}
	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)	
	client_cmd(id,"spk buttons/blip2")
	return PLUGIN_CONTINUE
}

public guide_rocket_rope(args[]){
	new aimvec[3],avgFactor
	new Float:fl_origin[3]
	new t_aimvec[33][3]
	new t_index[33]
	new t_jp,dist
	new jp_dist = 20000
	new id = args[0]
	new ent = args[1]
	new speed = args[2]
	new iNewVelocity[3]
	Entvars_Get_Vector(ent, EV_VEC_origin, fl_origin)
	new origin[3]
	origin[0] = floatround(fl_origin[0])
	origin[1] = floatround(fl_origin[1])
	origin[2] = floatround(fl_origin[2])

	if((cmd_onrope[args[6]] == 1) || (args[7] < 31)){
		if(is_user_alive(args[6]) == 1){
			get_user_origin(args[6],aimvec)
			dist = get_distance(aimvec,origin)
		}else{
			args[7] = 100
		}
		if(cmd_onrope[args[6]] == 1)
			args[7] = 0
	}else{
		new players[32], inum
		get_players(players,inum,"a")
		for(new i = 0 ;i < inum ;++i){
			if(players[i] != id){
				if(cmd_onrope[players[i]] == 1){
					new temp[3]
					get_user_origin(players[i],temp)
					t_aimvec[t_jp][0] = temp[0]
					t_aimvec[t_jp][1] = temp[1]
					t_aimvec[t_jp][2] = temp[2]
					t_index[t_jp] = players[i]
					t_jp++
				}
			}
		}
		for(new i = 0 ;i < t_jp ;++i){
			new temp[3]
			temp[0] = t_aimvec[i][0]
			temp[1] = t_aimvec[i][1]
			temp[2] = t_aimvec[i][2]
			dist = get_distance(temp,origin)
			if(dist < jp_dist){
				aimvec[0] = temp[0]
				aimvec[1] = temp[1]
				aimvec[2] = temp[2]
				jp_dist = dist
				args[6] = t_index[i]
				args[7] = 0	
			}
		}	
	}	  
	if(dist){
		if(speed < 400)
			avgFactor = 10
		else if(speed < 850)
			avgFactor = 4
		else
			avgFactor = 2

		new length, velocityvec[3]
		velocityvec[0]=aimvec[0]-origin[0] 
		velocityvec[1]=aimvec[1]-origin[1] 
		velocityvec[2]=aimvec[2]-origin[2] 
		length=sqrt(velocityvec[0]*velocityvec[0]+velocityvec[1]*velocityvec[1]+velocityvec[2]*velocityvec[2]) 
		velocityvec[0]=velocityvec[0]*speed/length 
		velocityvec[1]=velocityvec[1]*speed/length 
		velocityvec[2]=velocityvec[2]*speed/length 

		iNewVelocity[0] = (velocityvec[0] + (args[3] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[1] = (velocityvec[1] + (args[4] * (avgFactor-1) ) ) / avgFactor
		iNewVelocity[2] = (velocityvec[2] + (args[5] * (avgFactor-1) ) ) / avgFactor

		args[3] = iNewVelocity[0]
		args[4] = iNewVelocity[1]
		args[5] = iNewVelocity[2]
		if(dist < 60){
			vexd_pfntouch(ent,0)
			return PLUGIN_CONTINUE
		}			
	}
	args[7] += 1
	new Float:fl_iNewVelocity[3]
	fl_iNewVelocity[0] = args[3] +0.0
	fl_iNewVelocity[1] = args[4] +0.0
	fl_iNewVelocity[2] = args[5] +0.0

	Entvars_Set_Vector(ent, EV_VEC_velocity, fl_iNewVelocity)
	set_task(0.1,"guide_rocket_rope",2020+ent,args,16)

	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)	

	return PLUGIN_CONTINUE
}

public guide_rocket_swirl(args[]){
	new ent = args[1]
	set_task(0.1,"guide_rocket_swirl",2020+ent,args,16)
	new Float:missile_health
	missile_health = Float:Entvars_Get_Float(ent, EV_FL_health) 
	if(missile_health <10000.0)
		vexd_pfntouch(ent,0)

	return PLUGIN_CONTINUE
}

public rocket_fuel_timer(args[]){
	new ent = args[1]
	new id = args[0]
	remove_task(2020+ent)
	Entvars_Set_Int(ent, EV_INT_effects, 2)
	Entvars_Set_Int(ent, EV_INT_rendermode,0)
	Entvars_Set_Float(ent, EV_FL_gravity, 1.0)
	Entvars_Set_Int(ent, EV_INT_iuser1, 0) 
	emit_sound(ent, CHAN_WEAPON, "debris/beamstart8.wav", 1.0, ATTN_NORM, 0, PITCH_NORM )
	emit_sound(ent, CHAN_VOICE, "ambience/rocket_steam1.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
	if(args[11] == 1){
		set_hudmessage(255,0,0,-1.0,0.45, 0, 0.0, 1.5, 0.5, 0.15, 4)
		show_hudmessage(id,"WARNING:^nFUEL TANK EMPTY^nCONTROLS DISENGAGED") 
	}
	set_task(0.1,"guide_rocket_comm",2020+ent,args,16,"b")
	return PLUGIN_CONTINUE
}

public sqrt(num) { 
	new div = num 
	new result = 1 
	while (div > result) { 
		div = (div + result) / 2 
		result = num / div 
	} 
	return div 
} 

make_dot(vec[]){
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY)  
	write_byte( 17 ) 
	write_coord(vec[0]) 
	write_coord(vec[1]) 
	write_coord(vec[2])
	write_short( ls_dot ) 
	write_byte( 10 ) 
	write_byte( 255 ) 
	message_end()
}

public ls_missile_ck(){
	new arg[32]
	read_argv(1,arg,31)
	new iarg = strtonum(arg) 
	if(iarg){
		new szClassName[32]
		Entvars_Get_String(iarg, EV_SZ_classname, szClassName, 31)
		if(equal(szClassName, "lud_missile")) {
			new prize = get_cvar_num("amx_missile_prizes")
			new obey = get_cvar_num("amx_missile_obeyffcvar")
			new ff = get_cvar_num("mp_friendlyfire")
			new arg2[32],name[32],nameowner[32],Message[192]
			read_argv(2,arg2,31)
			new id = strtonum(arg2)
			new owner = Entvars_Get_Edict(iarg, EV_ENT_owner)
			new Tid,Towner
			if(dod_running){
				Tid = dod_team[id]
				Towner = dod_team[owner]
			}else{
				Tid = get_user_team(id)
				Towner = get_user_team(owner)
			}
			get_user_name(id,name,31)
			get_user_name(owner,nameowner,31)
			set_hudmessage(255,0,0, -1.0, 0.23, 0, 0.02, 5.0, 1.01, 1.1, 4)	
			if( (Tid == Towner) && (cstrike_running == true) && ( ((ff == 1 ) && (obey == 0)) || (ff == 0) ) ){
				format(Message,191,"Woohoo!!!!^n%s successfully shot down^n%s's missile with a laser",name,nameowner)
			}else{
				if(prize){
					format(Message,191,"Woohoo!!!!^n%s successfully shot down^n%s's missile with a laser^nand won free missiles!^n%d of each type!!!",name,nameowner,prize)
					client_print(id,print_chat,"[AMX] Congrats on shooting down that missile, you won free missiles")
					missile_win[id] += 1
					missile_inv[id][1] += prize
					missile_inv[id][2] += prize
					missile_inv[id][3] += prize
					missile_inv[id][4] += prize
					missile_inv[id][5] += prize
					missile_inv[id][6] += prize
					missile_inv[id][7] += prize
				}else{
					format(Message,191,"Woohoo!!!!^n%s successfully shot down^n%s's missile with a laser",name,nameowner)
				}
			}
			show_hudmessage(0,Message)
			client_cmd(0,"spk ^"rocket destroyed^"")
			client_cmd(0,"play ambience/lv4.wav")
			vexd_pfntouch(iarg,0)			
		}	
	}
	return PLUGIN_HANDLED
}	

public jp_missile_ck(){
	new sid[4],sstat[4]
	read_argv(1,sid,3)
	read_argv(2,sstat,3)
	new id = strtonum(sid)

	if(id < 1 || id > 32)
		id = 32
	cmd_jetpackrun[id] = strtonum(sstat)

//	new authid[32]
//	get_user_name(id,authid,31)
//	log_to_file("admin_logs/JP_DATA","M  id= %s   status= %s   authid= %s",sid,sstat,authid)

	return PLUGIN_CONTINUE
}

public rp_missile_ck(){
	new sid[4],sstat[4]
	read_argv(1,sid,3)
	read_argv(2,sstat,3)
	cmd_onrope[strtonum(sid)] = strtonum(sstat)
	return PLUGIN_CONTINUE
}

public HandleSay(id) {
	new Speech[192]
	read_args(Speech,192)
	remove_quotes(Speech)
	if( (containi(Speech, "rocket") != -1) || (containi(Speech, "missile") != -1) ){
		if(get_cvar_num("amx_luds_missiles") == 1){
			client_print(id,print_chat,"[AMX]  Missiles are enabled.  Help: say /missile_help  or say /missile")
		}else{
			client_print(id,print_chat, "[AMX]  Missles are disabled.")
		}
	}
	return PLUGIN_CONTINUE
}

public rocket_motd(id){
	new la_motd[1500]
	new temp[768]
	temp = "Either say /missile ..... or Bind a key to: missile_menu^n\
			 Go to your console ( press ~ ), choose a key for missiles.^n\
			 Lets say we want to use x ......  Do this in console:^n^n\
			 bind x missile_menu^n^n\
			 Enter that, and now whenever you press the x key, your^n\
			 missile menu pops up, simply chose a missile from menu^n"
	add(la_motd,1499,temp)
	temp = "and it fires. If you prefer not to use the menu and to make,^n\
			 your own binds, the direct console missile commands are:^n^n\
			 bind <choose a key> amx_missile^n\
			 bind <choose a key> amx_laserguided_missile^n\
			 bind <choose a key> amx_heatseeking_missile^n\
			 bind <choose a key> amx_guncamera_missile^n\
			 bind <choose a key> amx_anti_missile^n\
			 bind <choose a key> amx_ropeseeking_missile^n\
			 bind <choose a key> amx_swirlingdeath_missile^n\
			 amx_sdmf <swirling death fuel 10ths/second. Default: 20>^n^n"
	add(la_motd,1499,temp)
	temp = "Notes: Heatseeking missiles chase down the nearest running^n\
			 jetpack. Missiles by default only have 6 seconds of boost.^n\
			 Admins may set missiles to use money or for a limited number^n\
			 of missiles to be granted free each round in CS. Anti-^n\
			 missiles must be aimed at missile, then missile auto fires"
	add(la_motd,1499,temp)
	show_motd(id,la_motd,"Missile Help:")
	return PLUGIN_CONTINUE
}

show_missile_inv(id){
	if(get_cvar_num("amx_missile_buy") == 0){
		new Message[304]
		set_hudmessage(303,0,0, 0.80, 0.60, 0, 0.02, 6.0, 1.01, 1.1, 4)	
		if(get_cvar_num("amx_missile_ammo7takeall") == 0){
			format(Message,303,"MISSILE INVENTORY:^n^nCommon missile  ( %d )^nLaser guided missile  ( %d )^n\
				Heat seeking missile  ( %d )^nGun camera missile  ( %d )^nAnti Missile Shots  ( %d )^n\
				Rope seeking missile  ( %d )^nSwirling death missile  ( %d )",missile_inv[id][4],missile_inv[id][2],
				missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6],missile_inv[id][7])
		}else{
			new sum
			for(new i=1;i<8;i++)
				sum += missile_inv[id][i]	
			if(sum < 7){
				format(Message,303,"MISSILE INVENTORY:^n^nCommon missile  ( %d )^nLaser guided missile  ( %d )^n\
					Heat seeking missile  ( %d )^nGun camera missile  ( %d )^nAnti Missile Shots  ( %d )^n\
					Rope seeking missile  ( %d )^nSwirling death missile  ( 0 )",missile_inv[id][4],missile_inv[id][2],
					missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6])
			}else{
				format(Message,303,"MISSILE INVENTORY:^n^nCommon missile  ( %d )^nLaser guided missile  ( %d )^n\
					Heat seeking missile  ( %d )^nGun camera missile  ( %d )^nAnti Missile Shots  ( %d )^n\
					Rope seeking missile  ( %d )^nSwirling death missile  ( %d )",missile_inv[id][4],missile_inv[id][2],
					missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6],missile_inv[id][7])
			}
		}
		show_hudmessage(id,Message)
	}
	return PLUGIN_HANDLED
}

public show_main_menux(id){
	client_print(id,print_chat,"[AMX] OMG, its spelled ^"/missile^" you moron, but I'll help you anyway")
	show_main_menu(id)
	return PLUGIN_HANDLED
}

public show_main_menu(id){
	new menu_body[320]
	if (cstrike_running){
		if(get_cvar_num("amx_missile_buy") == 1){
			format(menu_body,319,"\yFire Missile Menu^n\w^n1. Common missile $%d^n2. Laser guided missile $%d^n\
			3. Heat seeking missile $%d^n4. Gun camera missile $%d^n5. Anti Missile Shots $%d^n\
			6. Rope seeking missile $%d^n7. Swirling death missile $%d^n^n8. Missile info window^n0. Exit",
			get_cvar_num("amx_missile_cost4"),get_cvar_num("amx_missile_cost2"),get_cvar_num("amx_missile_cost3"),
			get_cvar_num("amx_missile_cost1"),get_cvar_num("amx_missile_cost5"),get_cvar_num("amx_missile_cost6"),get_cvar_num("amx_missile_cost7"))
		}else{
			if(get_cvar_num("amx_missile_ammo7takeall") == 0){
				format(menu_body,319,"\yFire Missile Menu^n\w^n1. Common missile  ( %d )^n2. Laser guided missile  ( %d )^n\
				3. Heat seeking missile  ( %d )^n4. Gun camera missile  ( %d )^n5. Anti Missile Shots  ( %d )^n\
				6. Rope seeking missile  ( %d )^n7. Swirling death missile  ( %d )^n^n8. Missile info window^n0. Exit",missile_inv[id][4],
				missile_inv[id][2],missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6],missile_inv[id][7])
			}else{
				new sum
				for(new i=1;i<8;i++)
					sum += missile_inv[id][i]				
				if(sum < 7){
					format(menu_body,319,"\yFire Missile Menu^n\w^n1. Common missile  ( %d )^n2. Laser guided missile  ( %d )^n\
					3. Heat seeking missile  ( %d )^n4. Gun camera missile  ( %d )^n5. Anti Missile Shots  ( %d )^n\
					6. Rope seeking missile  ( %d )^n7. Swirling death missile  ( 0 )^n^n8. Missile info window^n0. Exit",missile_inv[id][4],
					missile_inv[id][2],missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6])
				}else{
					format(menu_body,319,"\yFire Missile Menu^n\w^n1. Common missile  ( %d )^n2. Laser guided missile  ( %d )^n\
					3. Heat seeking missile  ( %d )^n4. Gun camera missile  ( %d )^n5. Anti Missile Shots  ( %d )^n\
					6. Rope seeking missile  ( %d )^n7. Swirling death missile  ( %d )^n^n8. Missile info window^n0. Exit",missile_inv[id][4],
					missile_inv[id][2],missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6],missile_inv[id][7])
				}
			}
		}
	}else{
		format(menu_body,319,"Fire Missile Menu^n^n1. Common missile  ( %d )^n2. Laser guided missile  ( %d )^n\
		3. Heat seeking missile  ( %d )^n4. Gun camera missile  ( %d )^n5. Anti Missile Shots  ( %d )^n\
		6. Rope seeking missile  ( %d )^n7. Swirling death missile  ( %d )^n^n8. Missile info window^n0. Exit",missile_inv[id][4],
		missile_inv[id][2],missile_inv[id][3],missile_inv[id][1],missile_inv[id][5],missile_inv[id][6],missile_inv[id][7])
	}
	new a1,a2,a3,a4,a5,a6,a7,a8, keys = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<9)
	a1 = a2 = a3 = a4 = a5 = a6 = a7 = a8 = (cstrike_running ? 'd' : '#' )
	format(menu_body,255,menu_body,a1,a2,a3,a4,a5,a6,a7,a8)
	show_menu(id,keys,menu_body)
	return PLUGIN_HANDLED
}

public action_main_menu(id,key){
	using_menu[id] = 1
	switch(key){
		case 0: {
			client_cmd(id,"amx_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost4"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][4] < 1)
					show_main_menu(id)
			}
		}
		case 1: {
			client_cmd(id,"amx_laserguided_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost2"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][2] < 1)
					show_main_menu(id)
			}
		}  
		case 2: {
			client_cmd(id,"amx_heatseeking_missile") 
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost3"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][3] < 1)
					show_main_menu(id)
			}
		} 
		case 3: {
			client_cmd(id,"amx_guncamera_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost1"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][1] < 1)
					show_main_menu(id)
			}
		}
		case 4: {
			client_cmd(id,"amx_anti_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost5"))
 	 				show_main_menu(id)
			}else{
				if( (missile_inv[id][5] < 1) || (radar_batt[id] < 1) )
					show_main_menu(id)
			}
		}
		case 5: {
			client_cmd(id,"amx_ropeseeking_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost6"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][6] < 1)
					show_main_menu(id)
			}
		}
		case 6: {
			client_cmd(id,"amx_swirlingdeath_missile")
			if(get_cvar_num("amx_missile_buy") == 1){
				if(get_user_money(id) < get_cvar_num("amx_missile_cost7"))
 	 				show_main_menu(id)
			}else{
				if(missile_inv[id][7] < 1)
					show_main_menu(id)
			}
		}
		case 7: {
			client_cmd(id,"say /missile_help")
			using_menu[id] = 0
		}
		case 9: using_menu[id] = 0
	}
	return PLUGIN_HANDLED
}

public death_event(){
	new victim
	victim = read_data(2)
	remove_task(35632+victim)
	is_scan_rocket[victim] = 0	
}

public round_end(){
	if(get_cvar_num("amx_luds_missiles") == 0)
		return PLUGIN_CONTINUE
	if(DoOnce == 0){
		DoOnce = 1
		new maxplayers = get_maxplayers()+1
		for (new i=1; i<=maxplayers; i++) {
			if(is_user_connected(i) == 1){
				if( (is_user_alive(i) == 0) && (get_cvar_num("amx_missile_buy") == 1) ){
					missile_inv[i][1] = 0
					missile_inv[i][2] = 0
					missile_inv[i][3] = 0
					missile_inv[i][4] = 0
					missile_inv[i][5] = 0
					missile_inv[i][6] = 0
					missile_inv[i][7] = 0
				}
			}
			if(has_rocket[i] > 0)
				remove_missile(i,has_rocket[i])
		}
		round_st = 1
		set_task(20.0,"roundstart_delay")
	}
	return PLUGIN_CONTINUE
}

public plugin_log(){
	new args = read_logargc()
	if(args == 2){
		new msg[32]
		read_logargv(1,msg,31)
		if(msg[6] == 'S')
			round_start()
	}
	else if(args == 3){
		new msg[32]
		read_logargv(1,msg,31)
		if(equal(msg,"joined t",8)){
			new clinfo[64],won[32],cutoff,bcount
			new classinfo[32]	
			read_logargv(0,clinfo,63)
			read_logargv(2,classinfo,31)
			new tcbackw = strlen(clinfo)
			new cbackw = tcbackw
			for(new i=0;i<tcbackw;i++){
				cbackw -= 1
				if(cbackw > 0){
					if(clinfo[cbackw] == '<'){
						bcount +=1
						if(bcount == 3){
							copy(won,i-cutoff,clinfo[cbackw+1]) 
						}
						else if(bcount == 2)
							cutoff = i + 2						
					}
				}
			}
			new iwon = strtonum(won)
			new playerl = find_player("k",iwon)
			if(!playerl) return PLUGIN_CONTINUE
			if(equali(classinfo,"Axis"))
				dod_team[playerl] = 1
			else
				dod_team[playerl] = 2
		}	
	} 		
	return PLUGIN_CONTINUE
}

public round_start(){
	if(get_cvar_num("amx_luds_missiles") == 0)
		return PLUGIN_CONTINUE
	DoOnce = 0
	new prize = get_cvar_num("amx_missile_prizes")
	new maxplayers = get_maxplayers()+1
	for (new i=1; i<=maxplayers; i++) {
		radar_batt[i] = get_cvar_num("amx_missile_radarbattery")
		if(get_cvar_num("amx_missile_buy") == 0){
			missile_inv[i][1] = get_cvar_num("amx_missile_ammo1")
			missile_inv[i][2] = get_cvar_num("amx_missile_ammo2")
			missile_inv[i][3] = get_cvar_num("amx_missile_ammo3")
			missile_inv[i][4] = get_cvar_num("amx_missile_ammo4")
			missile_inv[i][5] = get_cvar_num("amx_missile_ammo5")
			missile_inv[i][6] = get_cvar_num("amx_missile_ammo6")
			missile_inv[i][7] = get_cvar_num("amx_missile_ammo7")
		}
		if(missile_win[i] > 0){
			missile_inv[i][1] += (prize * missile_win[i])
			missile_inv[i][2] += (prize * missile_win[i]) 
			missile_inv[i][3] += (prize * missile_win[i])
			missile_inv[i][4] += (prize * missile_win[i])
			missile_inv[i][5] += (prize * missile_win[i])
			missile_inv[i][6] += (prize * missile_win[i])
			missile_inv[i][7] += (prize * missile_win[i])
			missile_win[i] = 0
			client_print(i,print_chat,"[AMX] You won extra missiles from shooting down missile last round")
		}
	}
	new iCurrent = FindEntity(-1, "lud_missile") 
	while ((iCurrent = FindEntity(-1, "lud_missile")) != -1){ 
		new id = Entvars_Get_Edict(iCurrent, EV_ENT_owner)		
		remove_missile(id,iCurrent)
	}		
	return PLUGIN_CONTINUE
}

public roundstart_delay(){
	round_st = 0
}

remove_missile(id,missile){
	new Float:fl_origin[3]
	Entvars_Get_Vector(missile, EV_VEC_origin, fl_origin)
	
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(14) 
	write_coord(floatround(fl_origin[0])) 
	write_coord(floatround(fl_origin[1])) 
	write_coord(floatround(fl_origin[2]))  
	write_byte (200) 
	write_byte (40) 
	write_byte (45) 
	message_end()

	emit_sound(missile, CHAN_WEAPON, "ambience/particle_suck2.wav", 1.0, ATTN_NORM, 0, PITCH_NORM) 
	emit_sound(missile, CHAN_VOICE, "ambience/particle_suck2.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)		
	has_rocket[id] = 0
	is_heat_rocket[id] = 0
	is_scan_rocket[id] = 0
	remove_task(2020+missile)
	AttachView(id, id)
	RemoveEntity(missile)
}

#if use_newdeathmsg == 1
public stop_dm(id,tid,tbody,num){
	if(num == 9876){
		if (tbody == 1){
			message_begin( MSG_ALL, gmsgDeathMsg,{0,0,0},0)
			write_byte(id)
			write_byte(tid)
			write_string(" missile")
			message_end()
		}else{
			message_begin( MSG_ALL, gmsgDeathMsg,{0,0,0},0)
			write_byte(id)
			write_byte(tid)
			write_byte(0)
			write_string(" missile")
			message_end()
		}
	}else{
		if(redundant_plug == 0){
			new killer_id = read_data(1)
			new victim_id = read_data(2)
			new headshot = read_data(3)
			new weaponname[20]
			if(cstrike_running)
				read_data(4,weaponname,31)
			else
				read_data(3,weaponname,31)
			new weapon
			get_user_attacker(victim_id,weapon)
		
			if (!(killer_id==victim_id && !headshot && equal(weaponname,"world"))){
				message_begin( MSG_ALL, gmsgDeathMsg,{0,0,0},0)
				write_byte(killer_id)
				write_byte(victim_id)	
				write_byte(headshot)
				write_string(weaponname)
				message_end()
			}
		}
	}
	return PLUGIN_HANDLED
}

public regulate_plugins_dm(){
	new nump = get_pluginsnum()
	new filename[64],name[64],version[32],author[32],status[32],jit
	for(new i=0;i<nump;++i){ 			
		get_plugin(i,filename,63,name,63,version,31,author,31,status,31,jit)
		if(equali(filename,"warcraft3.amx")){
			set_cvar_num("amx_ejl_rpdm",1)
			redundant_plug = 1
		}
	}
	for(new i=0;i<nump;++i){ 			
		get_plugin(i,filename,63,name,63,version,31,author,31,status,31,jit)
		if(get_cvar_num("amx_ejl_rpdm")){
			if( (equali(filename,"amx_ejl_flamethrower.amx")) && (strtonum(version[0]) > 4) )
				redundant_plug = 1
			else if( (equali(filename,"cs_flamethrower.amx")) && (strtonum(version[0]) > 4) )
				redundant_plug = 1
			else if( (equali(filename,"amx_ejl_laserweapon.amx")) && (strtonum(version[0]) > 1) )
				redundant_plug = 1
			else if( (equali(filename,"laser_guns.amx")) && (strtonum(version[0]) > 1) )
				redundant_plug = 1
			else if( (equali(filename,"amx_ejl_timebomb.amx")) && (strtonum(version[0]) > 2) )
				redundant_plug = 1
			else if( (equali(filename,"kamikaze.amx")) && (strtonum(version[0]) > 2) )
				redundant_plug = 1
			else if( (equali(filename,"amx_ejl_jetpack.amx")) && (strtonum(version[0]) > 1) )
				redundant_plug = 1
			else if( (equali(filename,"luds_jetpack.amx")) && (strtonum(version[0]) > 1) )
				redundant_plug = 1
		}
	}
	set_cvar_num("amx_ejl_rpdm",1)
	set_task(1.0,"regulate_plugins_rs")
	return PLUGIN_CONTINUE
}

public regulate_plugins_rs(){
	set_cvar_num("amx_ejl_rpdm",0)
	return PLUGIN_CONTINUE
}
	
#endif

public SD_CircleRockets(Ent){ 
	new NUMBER_OF_ROCKETS = get_cvar_num("amx_missile_sdcount")
	new ROCKET_RADIUS =  get_cvar_num("amx_missile_sdradius")
	new Float:vOrigin[3] 
	new Float:vVelocity[3] 
	new Float:vEntOrig[3] 
	new Float:RotMatrix[3][3] 
	new Float:x, Float:y, Float:z, Float:theta 
	new i, id, NewEnt 
	Entvars_Get_Vector(Ent, EV_VEC_origin, vOrigin) 
	Entvars_Get_Vector(Ent, EV_VEC_velocity, vVelocity) 
	id = Entvars_Get_Edict(Ent, EV_ENT_owner) 

	RotMatrix[0][0]=vVelocity[0] 
	RotMatrix[0][1]=vVelocity[1] 
	RotMatrix[0][2]=vVelocity[2] 
	RotMatrix[1][0]=-vVelocity[1] 
	RotMatrix[1][1]=vVelocity[0] 
	RotMatrix[1][2]=0.0 
	SD_Normalise(RotMatrix[0]) 
	SD_Normalise(RotMatrix[1]) 

	RotMatrix[2][0]=((RotMatrix[0][1])*(RotMatrix[1][2]))-((RotMatrix[0][2])*(RotMatrix[1][1])) 
	RotMatrix[2][1]=((RotMatrix[0][2])*(RotMatrix[1][0]))-((RotMatrix[0][0])*(RotMatrix[1][2])) 
	RotMatrix[2][2]=((RotMatrix[0][0])*(RotMatrix[1][1]))-((RotMatrix[0][1])*(RotMatrix[1][0])) 

	for (i=0; i<NUMBER_OF_ROCKETS; i++){ 
		theta = (float(i)/float(NUMBER_OF_ROCKETS))*2*PI+g_fAngle 
		x = 0.0 
		y = cos(theta)*ROCKET_RADIUS 
		z = sin(theta)*ROCKET_RADIUS 

		vEntOrig[0]=RotMatrix[0][0]*x+RotMatrix[1][0]*y+RotMatrix[2][0]*z 
		vEntOrig[1]=RotMatrix[0][1]*x+RotMatrix[1][1]*y+RotMatrix[2][1]*z 
		vEntOrig[2]=RotMatrix[0][2]*x+RotMatrix[1][2]*y+RotMatrix[2][2]*z 
		vEntOrig[0]+=vOrigin[0] 
		vEntOrig[1]+=vOrigin[1] 
		vEntOrig[2]+=vOrigin[2] 
		NewEnt = SD_CreateRocket(vEntOrig, vVelocity, id) 
		Entvars_Set_Int(NewEnt, EV_INT_iuser1, Ent) 
		Entvars_Set_Int(NewEnt, EV_INT_iuser2, i) 
	} 
} 

public SD_CreateRocket(Float:vOrigin[3], Float:vVelocity[3], id) { 
	new Float:vAngles[3] 
	VecToAngles(vVelocity, vAngles) 
	new NewEnt 
	NewEnt = CreateEntity("info_target") 
	if(NewEnt == 0)
		return 0 
	Entvars_Set_String(NewEnt, EV_SZ_classname, "lud_missile") 
	ENT_SetModel(NewEnt, "models/rpgrocket.mdl") 
	new Float:fl_vecmins[3] = {-1.0, -1.0, -1.0}
	new Float:fl_vecmaxs[3] = {1.0, 1.0, 1.0}
	Entvars_Set_Vector(NewEnt, EV_VEC_mins, fl_vecmins) 
	Entvars_Set_Vector(NewEnt, EV_VEC_maxs, fl_vecmaxs) 
	ENT_SetOrigin(NewEnt, vOrigin) 
	Entvars_Set_Vector(NewEnt, EV_VEC_angles, vAngles) 
	Entvars_Set_Int(NewEnt, EV_INT_solid, 2)   
	Entvars_Set_Int(NewEnt, EV_INT_movetype, 6)  
	Entvars_Set_Edict(NewEnt, EV_ENT_owner, id) 
	Entvars_Set_Float(NewEnt, EV_FL_health, 10000.0)
	Entvars_Set_Float(NewEnt, EV_FL_takedamage, 100.0)
	Entvars_Set_Float(NewEnt, EV_FL_dmg_take, 100.0)
	Entvars_Set_Vector(NewEnt, EV_VEC_velocity, vVelocity) 
	make_trail(NewEnt,7) // 7 = swirling missiles
	Entvars_Set_Float(NewEnt, EV_FL_gravity, 0.000001 )

	emit_sound(NewEnt, CHAN_WEAPON, "weapons/rocketfire1.wav", 1.0, ATTN_NORM, 0, PITCH_NORM) 
	emit_sound(NewEnt, CHAN_VOICE, "weapons/rocket1.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
	new args[16]
	args[0] = id
	args[1] = NewEnt
	args[2] = get_cvar_num("amx_missile_sdspeed")
	args[3] = floatround(vVelocity[0])
	args[4] = floatround(vVelocity[1])
	args[5] = floatround(vVelocity[2])
	if( (user_sdmf[id] > 0.0) && (get_cvar_num("amx_missile_allow_sdmf") == 1) )
		set_task(user_sdmf[id],"rocket_fuel_timer",2020+NewEnt,args,16)
	else
		set_task(get_cvar_float("amx_missile_sdfuel"),"rocket_fuel_timer",2020+NewEnt,args,16)
	return NewEnt 
} 

public SD_Normalise(Float:Vector[3]){ 
	new Float:NullVector[3] = {0.0,0.0,0.0} 
	new Float:fLength = VecDist(Vector, NullVector) 
	Vector[0] /= fLength 
	Vector[1] /= fLength 
	Vector[2] /= fLength 
} 

public RocketThink() {
	new NUMBER_OF_ROCKETS = get_cvar_num("amx_missile_sdcount") 
	new ANGULAR_VELOCITY = get_cvar_num("amx_missile_sdrotate")
	new ROCKET_RADIUS =  get_cvar_num("amx_missile_sdradius") 
	new iCurrent, iCenterRocket, i 
	new Float:vOrigin[3], Float:vVelocity[3] 
	new Float:RotMatrix[3][3] 
	new Float:vEntOrig[3], Float:vOldEntOrig[3] 
	new Float:x, Float:y, Float:z, Float:theta 
	new Float:vAngles[3], Float:fNewVelocity[3] 
	new iTempEnt 
	new bool:bFound 
	g_fAngle += ANGULAR_VELOCITY * DT 

	if (g_fAngle>2*PI)
		g_fAngle -= 2*PI 

	iCurrent = FindEntity(-1, "lud_missile") 

	while (iCurrent != -1){ 
		iCenterRocket = Entvars_Get_Int(iCurrent, EV_INT_iuser1) 
		if (iCenterRocket){ 
			bFound = false 
			iTempEnt = -1 
			do{ 
				iTempEnt = FindEntity(iTempEnt, "lud_missile") 
				if (iTempEnt == iCenterRocket)
					bFound = true 
			}while (iTempEnt != -1 && !bFound)
			iTempEnt = -1
			do{ 
				iTempEnt = FindEntity(iTempEnt, "vexd_vrocket") 
				if (iTempEnt == iCenterRocket) 
					bFound = true 
			}while (iTempEnt != -1 && !bFound) 
			if (bFound){ 
				Entvars_Get_Vector(iCenterRocket, EV_VEC_origin, vOrigin) 
				Entvars_Get_Vector(iCenterRocket, EV_VEC_velocity, vVelocity) 
				Entvars_Get_Vector(iCurrent, EV_VEC_origin, vOldEntOrig) 
				i = Entvars_Get_Int(iCurrent, EV_INT_iuser2) 
				RotMatrix[0][0]=vVelocity[0] 
				RotMatrix[0][1]=vVelocity[1] 
				RotMatrix[0][2]=vVelocity[2] 
				RotMatrix[1][0]=-vVelocity[1] 
				RotMatrix[1][1]=vVelocity[0] 
				RotMatrix[1][2]=0.0 
				SD_Normalise(RotMatrix[0]) 
				SD_Normalise(RotMatrix[1]) 
				RotMatrix[2][0]=((RotMatrix[0][1])*(RotMatrix[1][2]))-((RotMatrix[0][2])*(RotMatrix[1][1])) 
				RotMatrix[2][1]=((RotMatrix[0][2])*(RotMatrix[1][0]))-((RotMatrix[0][0])*(RotMatrix[1][2])) 
				RotMatrix[2][2]=((RotMatrix[0][0])*(RotMatrix[1][1]))-((RotMatrix[0][1])*(RotMatrix[1][0])) 
				theta = (float(i)/float(NUMBER_OF_ROCKETS))*2*PI+g_fAngle 
				x = 0.0 
				y = cos(theta)*ROCKET_RADIUS 
				z = sin(theta)*ROCKET_RADIUS 
				vEntOrig[0]=RotMatrix[0][0]*x+RotMatrix[1][0]*y+RotMatrix[2][0]*z 
				vEntOrig[1]=RotMatrix[0][1]*x+RotMatrix[1][1]*y+RotMatrix[2][1]*z 
				vEntOrig[2]=RotMatrix[0][2]*x+RotMatrix[1][2]*y+RotMatrix[2][2]*z 
				vEntOrig[0]+=vOrigin[0] 
				vEntOrig[1]+=vOrigin[1] 
				vEntOrig[2]+=vOrigin[2] 
				vEntOrig[0]+=vVelocity[0]*DT 
				vEntOrig[1]+=vVelocity[1]*DT 
				vEntOrig[2]+=vVelocity[2]*DT 
				CalculateVelocity(vOldEntOrig, vEntOrig, fNewVelocity) 
				Entvars_Set_Vector(iCurrent, EV_VEC_velocity, fNewVelocity) 
				VecToAngles(fNewVelocity, vAngles) 
				Entvars_Set_Vector(iCurrent, EV_VEC_angles, vAngles)
				new Float:missile_health
				missile_health = Float:Entvars_Get_Float(iCurrent, EV_FL_health) 
				if(missile_health <10000.0)
					vexd_pfntouch(iCurrent,0)  
			}else{ 
				Entvars_Set_Int(iCurrent, EV_INT_iuser1, 0) 
			} 
		} 
		iCurrent =  FindEntity(iCurrent, "lud_missile") 
	} 
} 

public CalculateVelocity(Float:vOrigin[3], Float:vEnd[3], Float:vVelocity[3]){ 
	vVelocity[0] = (vEnd[0] - vOrigin[0]) / DT 
	vVelocity[1] = (vEnd[1] - vOrigin[1]) / DT 
	vVelocity[2] = (vEnd[2] - vOrigin[2]) / DT 
}

public new_spawn(id){
	#if use_newdeathmsg == 1
	set_msg_block(gmsgDeathMsg, BLOCK_SET)
	#endif
	return PLUGIN_CONTINUE
}		

public plugin_precache() {
	precache_sound("vox/_period.wav")
	precache_sound("debris/beamstart8.wav")
	precache_sound("weapons/explode3.wav")
	precache_sound("weapons/rocketfire1.wav") 
	precache_sound("ambience/rocket_steam1.wav") 
	precache_sound("weapons/rocket1.wav")
	precache_sound("ambience/particle_suck2.wav") 
	precache_model("models/rpgrocket.mdl")
//	precache_model("models/hvr.mdl")
	if(cstrike_running)
		precache_model("models/chick.mdl")
	beam = precache_model("sprites/smoke.spr")
	boom = precache_model("sprites/zerogxplode.spr")
	ls_dot = precache_model("sprites/laserdot.spr")
	return PLUGIN_CONTINUE 
}

public plugin_init(){ 
	register_plugin("CS Missiles","3.7.4","EJL")
	register_concmd("amx_missiles","admin_missiles",ADMIN_LEVEL_H,": toggles missiles mode on and off")
	register_concmd("amx_missilesbuying","admin_missilebuy",ADMIN_LEVEL_H,": toggles missile buy requirement mode on and off") 
	register_concmd("amx_missiles_ammo1","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : guncam missiles given per round free if missile buying is off")
	register_concmd("amx_missiles_ammo2","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : laserguide missiles given per round free if missile buying is off")
	register_concmd("amx_missiles_ammo3","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : heatseek missiles given per round free if missile buying is off")
	register_concmd("amx_missiles_ammo4","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : common missiles given per round free if missile buying is off")
	register_concmd("amx_missiles_ammo5","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : anti missile shots given per round free if missile buying is off")
	register_concmd("amx_missiles_ammo6","admin_missile_ammo",ADMIN_LEVEL_H,"<quantity> : ropeseek missiles given per round free if missile buying is off")
	register_concmd("amx_missile1","admin_missile",ADMIN_LEVEL_D,"<speed=1000> <chicken=0> <deadly=0> : guncam style")
	register_concmd("amx_missile2","admin_missile",ADMIN_LEVEL_D,"<speed=1000> <chicken=0> <deadly=0> : laserguided style")
	register_concmd("amx_missile3","admin_missile",ADMIN_LEVEL_D,"<speed=1000> <chicken=0> <deadly=0> : heatseeking style")
	register_concmd("amx_missile4","admin_missile",ADMIN_LEVEL_D,"<speed=1000> <chicken=0> <deadly=0> : common style")
	register_clcmd("amx_guncamera_missile","fire_missile")
	register_clcmd("amx_laserguided_missile","fire_missile")
	register_clcmd("amx_heatseeking_missile","fire_missile")
	register_clcmd("amx_missile","fire_missile")
	register_clcmd("amx_anti_missile","fire_missile")
	register_clcmd("amx_ropeseeking_missile","fire_missile")
	register_clcmd("amx_swirlingdeath_missile","fire_missile")
	register_concmd("amx_missile7","admin_missile",ADMIN_LEVEL_D,"<speed=1000> <chicken=0> <deadly=0> : swrirling death style")
	register_clcmd("amx_sdmf","player_sdmf")
	register_clcmd("say", "HandleSay")
	register_clcmd("say /missile_help","rocket_motd")
	register_clcmd("say /missile","show_main_menu")
	register_clcmd("say /missle","show_main_menux")
	register_clcmd("say /missl","show_main_menux")
	register_clcmd("say /misle","show_main_menux")
	register_clcmd("say missile","show_main_menux")
	register_clcmd("say missle","show_main_menux")
	register_clcmd("say missl","show_main_menux")
	register_clcmd("say misle","show_main_menux")
	register_clcmd("missile_menu", "show_main_menu")	
	register_menucmd(register_menuid("Fire Missile Menu"),1023,"action_main_menu")
	register_srvcmd("lasermissile_chk","ls_missile_ck")
	register_srvcmd("jetpackmissile_chk","jp_missile_ck")
	register_srvcmd("ropemissile_chk","rp_missile_ck")
	register_srvcmd("bot_missile","bot_interface")
	register_cvar("amx_luds_missiles","1",FCVAR_SERVER)
	set_cvar_string("amx_luds_missiles","1")
	register_cvar("amx_missile_buy","0")
	set_cvar_string("amx_missile_buy","0")
	register_cvar("amx_missile_fuel","6.0")
	set_cvar_float("amx_missile_fuel",6.0)
	register_cvar("amx_missile_sdfuel","2.0")
	set_cvar_float("amx_missile_sdfuel",2.0)
	register_cvar("amx_missile_sdcount","6")
	set_cvar_string("amx_missile_sdcount","6")
	register_cvar("amx_missile_sdrotate","6")
	set_cvar_string("amx_missile_sdrotate","6")
	register_cvar("amx_missile_sdradius","32")
	set_cvar_string("amx_missile_sdradius","32")
	register_cvar("amx_missile_allow_sdmf","0")
	set_cvar_string("amx_missile_allow_sdmf","0")
	register_cvar("amx_missile_ammo1","1")
	set_cvar_string("amx_missile_ammo1","1")
	register_cvar("amx_missile_ammo2","1")
	set_cvar_string("amx_missile_ammo2","1")
	register_cvar("amx_missile_ammo3","1")
	set_cvar_string("amx_missile_ammo3","1")
	register_cvar("amx_missile_ammo4","2")
	set_cvar_string("amx_missile_ammo4","2")
	register_cvar("amx_missile_ammo5","1")
	set_cvar_string("amx_missile_ammo5","1")
	register_cvar("amx_missile_ammo6","1")
	set_cvar_string("amx_missile_ammo6","1")
	register_cvar("amx_missile_ammo7","1")
	set_cvar_string("amx_missile_ammo7","1")
	register_cvar("amx_missile_ammo7takeall","0")
	set_cvar_string("amx_missile_ammo7takeall","0")
	register_cvar("amx_missile_cost1","3000")
	set_cvar_string("amx_missile_cost1","3000")
	register_cvar("amx_missile_cost2","3000")
	set_cvar_string("amx_missile_cost2","3000")
	register_cvar("amx_missile_cost3","4000")
	set_cvar_string("amx_missile_cost3","4000")
	register_cvar("amx_missile_cost4","1000")
	set_cvar_string("amx_missile_cost4","1000")
	register_cvar("amx_missile_cost5","2000")
	set_cvar_string("amx_missile_cost5","2000")
	register_cvar("amx_missile_cost6","4000")
	set_cvar_string("amx_missile_cost6","4000")
	register_cvar("amx_missile_cost7","5000")
	set_cvar_string("amx_missile_cost7","5000")
	register_cvar("amx_missile_speed","1000")
	set_cvar_string("amx_missile_speed","1000")
	register_cvar("amx_missile_sdspeed","750")
	set_cvar_string("amx_missile_sdspeed","750")
	register_cvar("amx_missile_hsspeed","1100")
	set_cvar_string("amx_missile_hsspeed","1100")
	register_cvar("amx_missile_rsspeed","1400")
	set_cvar_string("amx_missile_rsspeed","1400")
	register_cvar("amx_missile_obeygravity","1")
	set_cvar_string("amx_missile_obeygravity","1")
	register_cvar("amx_missile_damradius1","90")
	set_cvar_string("amx_missile_damradius1","90")
	register_cvar("amx_missile_damradius2","160")
	set_cvar_string("amx_missile_damradius2","160")
	register_cvar("amx_missile_damradius3","270")
	set_cvar_string("amx_missile_damradius3","270")
	register_cvar("amx_missile_radiusdam1","100")
	set_cvar_string("amx_missile_radiusdam1","100")
	register_cvar("amx_missile_radiusdam2","66")
	set_cvar_string("amx_missile_radiusdam2","66")
	register_cvar("amx_missile_radiusdam3","25")
	set_cvar_string("amx_missile_radiusdam3","25")
	register_cvar("amx_missile_obeyffcvar","1")
	set_cvar_string("amx_missile_obeyffcvar","1")
	register_cvar("amx_missile_tkpunish1","1")
	set_cvar_string("amx_missile_tkpunish1","1")
	register_cvar("amx_missile_tkpunish2","3")
	set_cvar_string("amx_missile_tkpunish2","3")
	register_cvar("amx_missile_prizes","5")
	set_cvar_string("amx_missile_prizes","5")
	register_cvar("amx_missile_radarbattery","100")
	set_cvar_string("amx_missile_radarbattery","100")
	register_cvar("amx_missile_botsnolimit","1")
	set_cvar_string("amx_missile_botsnolimit","1")
	register_cvar("amx_missile_atspawn","0")
	set_cvar_string("amx_missile_atspawn","0")
	register_cvar("amx_missile_conmsg","0")
	set_cvar_string("amx_missile_conmsg","0")
	register_event("SendAudio","round_end","a","2=%!MRAD_terwin","2=%!MRAD_ctwin","2=%!MRAD_rounddraw")
	register_event("TextMsg","round_end","a","2&#Game_C","2&#Game_w")
	register_event("DeathMsg","death_event","a")
	register_event("ResetHUD", "new_spawn", "b")	
	new mod_name[32]
	get_modname(mod_name,31)
	cstrike_running = equal(mod_name,"cstrike") ? true : false
	dod_running = equal(mod_name,"dod") ? true : false
	#if use_newdeathmsg == 1
	gmsgDeathMsg = get_user_msgid("DeathMsg")
	register_event("DeathMsg", "stop_dm", "a")
	set_msg_block(gmsgDeathMsg, BLOCK_SET)
	set_task(1.0,"regulate_plugins_dm")
	register_cvar("amx_ejl_rpdm","0")
	#endif
	set_task(0.1,"RocketThink",30500,"",0,"b")
	return PLUGIN_CONTINUE 
}
