/* AMX Mod script.
*
* Advertising Filter 1.1
*  by SniperBeamer
*
*
*  Description:
*   Blocks and logs advertising (blacklist and ips).
*   You can set up a whitelist and a blacklist.
*
*  Configuration:
*   amx_adfilter_allow_admins - allows admins to advertise (default: 1)
*
* This file is provided as is (no warranties).
*
*/

#include <amxmod>
#include <amxmisc>

#define MAX_ENTRIES 50

new g_logFile[16]
new g_WhiteList[MAX_ENTRIES][32]
new g_WhiteEntries
new g_BlackList[MAX_ENTRIES][32]
new g_BlackEntries

public plugin_init()
{
	register_plugin("Advertising Filter","1.1","SniperBeamer")
	register_cvar("amx_adfilter_allow_admins","1")
	register_clcmd("say","check_ad")
	register_clcmd("say_team","check_ad")

	new filename[64]
	build_path(filename,63,"$basedir/whitelist.ini")
	load_whitelist(filename)
	build_path(filename,63,"$basedir/blacklist.ini")
	load_blacklist(filename)

	get_logfile(g_logFile,15)
}

bool:is_ip(said[])
{
	new pos = -1,pos2 = 0

	for (new i=0; i<3; i++)
	{
		pos2 += pos + 1
		pos = contain(said[pos2],".")

		if (pos==-1)
			return false
		if ( !isdigit(said[pos-1+pos2]) || !isdigit(said[pos+1+pos2]) )
			return false
	}

	return true
}

bool:is_admin(id)
{
	new flags = get_user_flags(id)
	if (
		(flags&ADMIN_IMMUNITY) ||
		(flags&ADMIN_RESERVATION) ||
		(flags&ADMIN_KICK) ||
		(flags&ADMIN_BAN) ||
		(flags&ADMIN_SLAY) ||
		(flags&ADMIN_MAP) ||
		(flags&ADMIN_CVAR) ||
		(flags&ADMIN_CFG) ||
		(flags&ADMIN_CHAT) ||
		(flags&ADMIN_VOTE) ||
		(flags&ADMIN_PASSWORD) ||
		(flags&ADMIN_RCON) ||
		(flags&ADMIN_LEVEL_A) ||
		(flags&ADMIN_LEVEL_B) ||
		(flags&ADMIN_LEVEL_C) ||
		(flags&ADMIN_LEVEL_D) ||
		(flags&ADMIN_LEVEL_E) ||
		(flags&ADMIN_LEVEL_F) ||
		(flags&ADMIN_LEVEL_G) ||
		(flags&ADMIN_LEVEL_H) ||
		(flags&ADMIN_MENU)
	   )
	{
		return true
	}

	return false
}

trim2(string[])
{
	while (contain(string," ")!=-1)
		replace(string,strlen(string)," ","")
	while (contain(string,"^t")!=-1)
		replace(string,strlen(string),"^t","")
}

public check_ad(id)
{
	new said[256],bool:bad = false
	read_args(said,255)

	for (new i=0; i<g_BlackEntries; i++)
	{
		if (containi(said,g_BlackList[i])!=-1)
		{
			bad = true
			break
		}
	}

	if (bad || is_ip(said))
	{
		if ( get_cvar_num("amx_adfilter_allow_admins") && is_admin(id) ) return PLUGIN_CONTINUE
		for (new i=0; i<g_WhiteEntries; i++)
			if (containi(said,g_WhiteList[i])!=-1) return PLUGIN_CONTINUE

		new name[32],authid[32]
		get_user_name(id,name,31)
		get_user_authid(id,authid,31)

		client_print(id,print_chat,"No advertising, please!")
  		log_to_file(g_logFile,"Advertising: ^"%s<%d><%s><>^" said %s", 
			name,get_user_userid(id),authid,said )

		return PLUGIN_HANDLED
	}

	return PLUGIN_CONTINUE
}

load_whitelist(filename[])
{
	if (!file_exists(filename)) return

	new text[64],text2[64]
	new a,pos = 0

	while ( g_WhiteEntries < MAX_ENTRIES && read_file(filename,pos++,text,255,a) )
	{
		if (text[0]==';') continue

		copy(text2,63,text)
		trim2(text2)
		if (equal(text2,"")) continue

		copy(g_WhiteList[g_WhiteEntries],31,text)

		g_WhiteEntries++
	}
}

load_blacklist(filename[])
{
	if (!file_exists(filename)) return

	new text[64],text2[64]
	new a,pos = 0

	while ( g_BlackEntries < MAX_ENTRIES && read_file(filename,pos++,text,255,a) )
	{
		if (text[0]==';') continue

		copy(text2,63,text)
		trim2(text2)
		if (equal(text2,"")) continue

		copy(g_BlackList[g_BlackEntries],31,text)

		g_BlackEntries++
	}
}
