/*
* hud compass by jon
*
* this file contains code to add a compass to
* the HUD of a cs player.
*
* hud_compass <0|1> - disable|enable
*
* the compass displays the general cardinal
* direction the player is facing, the distance to
* the point they are aiming at, and the relative
* elevation. the server will poll this information
* each client id that has asked for it, every second,
* until they disable it or disconnect.
*
* theoretically, this plugin increases the efficacy of
* voice-communication software that people otherwise
* use for taunts, bragging, day-to-day chatting, etc.
* (while the rest of us are trying to fucking play). it
* also provides a little help in learning maps, such as
* the huge ones in natural selection with hud location
* displays. i haven't tested it out with ns, though.
*
* TODO:
*
*  more units? what do people use other than meters and feet?
*
*  option to display direction as a bearing.
*    e.g., N 1010'10" E, S 10500'04" W, NAZ 00000'00"
*    foo(x, "%s %d%02d'%02d^" %s", major,deg,min,sec,minor)
*
*/

#include <amxmod>
#include <amxmisc>

// types
enum units { meters, feet, yards, generic }

// consts
static const CURR_VERSION[] = "1.6"

// no-modify vars
new bool:display_compass[32] = { false, ... }
new units:display_units[32]
new display_colors[32][3]
static Float:unit_divisors[4] = { 42.0, 12.0, 36.0, 1.0 }
static unit_denoters[4][3] = { "m", "ft", "yd", "u" }
static unit_names[4][8] = { "meters", "feet", "yards", "generic" }


get_bearing(Float:deg_angle, c[])
{
	new d, m, s
	new Float:fd, Float:fm, Float:fs, Float:deg = deg_angle

	fd = float(floatround(deg, floatround_floor))
	deg = (deg - fd) * 60.0
	fm = float(floatround(deg, floatround_floor))
	deg = (deg - fm) * 60.0
	fs = float(floatround(deg, floatround_floor))

	d = floatround(fd, floatround_floor)
	m = floatround(fm, floatround_floor)
	s = floatround(fs, floatround_floor)

	format(c, 14, "%d' %02d' %02d^"", d, m, s)
	//format(c, 14, "%d-%02d-%02d", d, m, s)
}



public toggle_compass(id)
{
	new bool:nochange = false
	new uarg[1]
	read_argv(1, uarg, 1)

	if (equal(uarg,"1")) display_compass[id-1] = true
	else if (equal(uarg,"0")) display_compass[id-1] = false
	else nochange = true

	console_print(id, "compass %s %sabled.", (nochange ? "is currently" : "has been"), (display_compass[id-1] ? "en" : "dis"))

	return PLUGIN_HANDLED
}


public change_units (id)
{
	new bool:nochange = false
	new uarg[2]
	read_argv(1, uarg, 1)

	if (equal(uarg,"3") || equali(uarg,"g") || equali(uarg,"u")) display_units[id-1] = generic
	if (equal(uarg,"2") || equali(uarg,"y")) display_units[id-1] = yards
	else if (equal(uarg,"1") || equali(uarg,"f")) display_units[id-1] = feet
	else if (equal(uarg,"0") || equali(uarg,"m")) display_units[id-1] = meters
	else nochange = true

	console_print(id, "compass units %s: %s.", (nochange ? "are in" : "set to"), unit_names[display_units[id-1]])
	return PLUGIN_HANDLED
}


public change_colors (id,level,cid)
{
	new ared[4], agreen[4], ablue[4]
	new tnred, tngreen, tnblue

	read_argv(1, ared, 3)
	read_argv(2, agreen, 3)
	read_argv(3, ablue, 3)

	tnred = strtonum(ared)
	tngreen = strtonum(agreen)
	tnblue = strtonum(ablue)

	if (tnred < 0) tnred = 0
	if (tngreen < 0) tngreen = 0
	if (tnblue < 0) tnblue = 0
	if (tnred > 255) tnred = 255
	if (tngreen > 255) tngreen = 255
	if (tnblue > 255) tnblue = 255

	display_colors[id-1][0] = tnred
	display_colors[id-1][1] = tngreen
	display_colors[id-1][2] = tnblue

	return PLUGIN_HANDLED
}


public compass_poll ()
{
	new x
	new origin[3], facing[3], bearing[14], cardinal[20], major[2], minor[2]
	new Float:delta_x, Float:delta_y, Float:delta_z, Float:xydist, Float:at_deg

	for(x = 0; x < 32; x++)
	{
		/* id = x+1 */

		if (display_compass[x])
		{
			if (!is_user_connected(x+1))
			{
				display_compass[x] = false
				continue
			}

			get_user_origin(x+1, origin, 1)
			get_user_origin(x+1, facing, 3)
			xydist = float(get_distance(origin, facing))
			delta_x = float(facing[0] - origin[0])
			delta_y = float(facing[1] - origin[1])
			delta_z = float(facing[2] - origin[2])

			if (delta_x == 0.0)
			{
				cardinal = "SAZ 0000'00 "
				if (facing[1] > origin[1]) cardinal = "NAZ 0000'00 "
			}
			else if (delta_y == 0.0)
			{
				cardinal = "WAZ 0000'00 "
				if (facing[0] > origin[0]) cardinal = "EAZ 0000'00 "
			}
			else
			{
				at_deg = ( 180.0 / M_PI ) * atan2(delta_y, delta_x)
				if (at_deg < 0.0)  at_deg *= -1.0
				if (at_deg > 90.0) at_deg -= 90.0
				major = (delta_y > 0) ? "N" : "S"
				minor = (delta_x > 0) ? "E" : "W"
				get_bearing(at_deg, bearing)
				format(cardinal, 20, "%s %s %s", major, bearing, minor)
			}

			set_hudmessage(display_colors[x][0], display_colors[x][1], display_colors[x][2],
					-0.89, -0.98, 0, 0.0, 0.8, 0.1, 0.1, 69)

			show_hudmessage(x+1, "[ %s ]  dist: %0.2f%s  elev: %0.2f%s", cardinal,
					floatdiv(xydist,  unit_divisors[display_units[x]]), unit_denoters[display_units[x]],
					floatdiv(delta_z, unit_divisors[display_units[x]]), unit_denoters[display_units[x]]
					)
		}
	}

	return PLUGIN_CONTINUE
}


public plugin_init ()
{
	/*
	  small is retarded and won't let me do:
	    new units:display_units[32] = { meters, ... }
	  or:
	    new display_colors[32][3] = { {12, 255, 12}, ... }
	*/
	new i
	for (i = 0; i < 32; i++)
	{
		display_units[i] = meters;
		display_colors[i] = {12, 255, 12};
	}

	register_plugin("hud_compass", CURR_VERSION, "jon")
	register_concmd("hud_compass", "toggle_compass", ADMIN_USER, "<0|1> toggle compass on/off")
	register_concmd("hud_compass_units", "change_units", ADMIN_USER, "<0-2> set units")
	register_concmd("hud_compass_color", "change_colors", ADMIN_USER, "<0-255> <0-255> <0-255> set color")
	set_task(0.0, "compass_poll", 0, "", 0, "ab", 1)
}

