
/***************************************************************************
 *  amx_ejl_mapcvars.sma
 *   version 1.1        Date: 12/31/2002 - 2/23/2003
 *   Author: Eric Lidman     ejlmozart@hotmail.com
 *   Alias: Ludwig van       Upgrade: http://lidmanmusic.com/cs/plugins.html
 *
 *  Plugin allows you set server cvars that are particular to a map using a
 *   file in your ..addons/amx folder called map_cvars.txt. You should know
 *   this already, but I will say it anyway -- to set cvars for all maps, 
 *   use the server.cfg instead.
 *
 *  Command:
 *
 *    amx_lockmapcvars  -- turns on and off the locking of cvars listed with 
 *                         a "1" in the map_cvars.txt. If no cvars are listed
 *                         this command has no effect. If cvars are listed
 *                         with a "1", then they are by default locked 
 *                         unless this command is issued. If they have a "0"
 *                         then this command has no effect. ADMIN_RCON access
 *                         is required to use this command.
 *           
 *  The format of the cvar file: M = map line   C = cvar line   0|1 = locked 
 *   M  mapname
 *   C  lock0|1  cvar  value
 *  
 *   Everytime you give a mapname, you must precced it with "Map". Then the
 *   line or lines below must start with "C" indicating you are setting a 
 *   cvar. After the "C", leave a space, then put a number "1" or "0". "1"
 *   locks out any changes from votes or by lower admins. Note that ADMIN_RCON
 *   level admins can change this value in game for the remainder of the map
 *   time using amx_lockmapcvars. The values will not be changed in the file
 *   so next time this map is player, the value will be that which is in the
 *   file. "0" allows for changes to take place. So a "0" will set the default
 *   for map start, but leaves the cvar soft and changeable by admins or by
 *   voting, a "1" will set the default for a map and keeps it that way unless
 *   an RCON leve admin changes it. You may have as many cvars as you want. 
 *   Once the next line is "Map" or end of file, you are done setting cvars 
 *   for that map. You may then continue to repeat the process for each map 
 *   you want to set special cvars for. Updated. If you want to lock cvars for
 *   all maps, make the map name "*allmaps", then list your cvars as you would
 *   with individual maps. All maps lock should be listed first in the file.
 *
 *  Actual example of the contents of a working map_cvars.txt:
 *
 *   Map  *allmaps
 *   C 1  mp_hostagepenalty 0
 *   Map  oc_bestjump3
 *   C 1  sv_gravity 1000
 *   Map  starwars
 *   C 1  sv_gravity 100
 *   C 1  sv_airaccelerate -15
 *   C 1  mp_friendlyfire 0
 *   Map  de_dust
 *   C 0  mp_friendlyfire 1
 *
 ****************************************************************************/

#include <amxmod>
#include <amxmisc>

new bool:cvar_lock = true
new inum_cvars
new locked_cvar[25][64]
new locked_value[25][32]
new locked_yn[25]


public plugin_init() { 
	register_plugin("MAP CVARs","0.9","EJL")
	register_concmd("amx_lockmapcvars","lock_unlock",ADMIN_RCON,"amx_lockmapcvars : Locks/unlocks whatever cvars you set in map_cvars.txt")
	set_task(1.0,"set_map_cvars")
	set_task(5.1,"check_map_cvars",1721,"",0,"a", 9999)
	return PLUGIN_CONTINUE
}

public set_map_cvars(){ 
	new ThisMap[33]
	get_mapname(ThisMap, 33)
	new bool:map_on = false

	if(file_exists("addons/amx/map_cvars.txt") == 1){
		new line = 0
		new data[128]
		new stextsize = 0
		new type[4]
		new map_or_lock[32]
		new map_cvar[64]
		new value[32]
		log_message("[AMX][MAPCVARS] Cvar execution begin.")
		while((line=read_file("addons/amx/map_cvars.txt",line,data,128,stextsize))!=0){
			parse(data,type,4,map_or_lock,32,map_cvar,64,value,32)
			if(equali(type,"M",1)){
				if(map_on == true){
					map_on = false
				}
				if( (equali(map_or_lock,ThisMap)) || (equali(map_or_lock,"*allmaps")) ){
					map_on = true
				}
			}
			else if(equali(type,"C",1)){
				if(map_on == true){
					copy(locked_cvar[inum_cvars],64,map_cvar)
					copy(locked_value[inum_cvars],32,value)
					if(equal(map_or_lock,"1"))
						locked_yn[inum_cvars] = 1
					server_cmd("%s %s",map_cvar,value)
					log_message("[AMX][MAPCVARS] Executed:  %s %s",map_cvar,value)
					inum_cvars +=1
					if(inum_cvars >= 25){
						map_on = false
						log_message("[AMX][MAPCVARS]  Limit of 25 special cvars reached.")
					}
				}
			}
		}
		if(inum_cvars < 1)
			log_message("[AMX][MAPCVARS] Map_cvars.txt contains no cvars for this map.")	
		log_message("[AMX][MAPCVARS] Cvar execution end.")		
	}else{
		write_file("addons/amx/map_cvars.txt","*** MAP CVAR FILE. SEE README. EXAMPLE BELOW:  M mapname   C cvar   0|1 Lock",-1)
 		write_file("addons/amx/map_cvars.txt","Map oc_bestjump3",-1)
 		write_file("addons/amx/map_cvars.txt","C 0 sv_gravity 800",-1)
	}
	return PLUGIN_HANDLED
}

public check_map_cvars(){
	if(cvar_lock == true){
		for(new i = 0 ;i < inum_cvars; ++i){
			new cvar[64]
			if(strlen(locked_cvar[i]) != 0){
				get_cvar_string(locked_cvar[i],cvar,64)
				if(!equal(cvar,locked_value[i])){
					if(locked_yn[i] == 1){
						server_cmd("%s %s",locked_cvar[i],locked_value[i])
						log_message("[AMX][MAPCVARS] Attemped change of locked cvar detected. Returning %s to %s",locked_cvar[i],locked_value[i])
						client_print(0,print_chat,"[AMX] Attemped change of locked cvar detected. Returning %s to %s",locked_cvar[i],locked_value[i])
					}else{
						if(locked_yn[i] == 0)
							log_message("[AMX][MAPCVARS] Change of locked cvar detected, but change is allowed.")
						locked_yn[i] = 2
					}
				}
			}
		}
	}
	return PLUGIN_CONTINUE
}

public lock_unlock(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	if (cvar_lock == true){
		cvar_lock = false
		client_print(id,print_console,"[AMX] Map Cvar lock is OFF.")
	}else	{
		cvar_lock = true
		client_print(id,print_console,"[AMX] Map Cvar lock is ON.")
	}
	return PLUGIN_HANDLED
}

