
/* AMX Mod script. 
*
* tracer_fire 1.3 by jon
*
* (c) Copyright 2002, OLO 
* This file is provided as is (no warranties). 
*
* Modified by Hephaistos from amxmod.net 
* (original plugin is "Weapon Tracers")
* to draw tracers on every bullet.
*
* Modified yet again by shusta <jon@wroth.org>:
*  - originally "war_weapon_tracers.amx"
*
* bugfixes from war_weapon_tracers:
*  - doesn't trigger on weapon change
*
* now featuring:
*  - GAY-AS-PINK-SUNSHINE LASERBEAM TRACER(TM)
*  - weapon-based colors (see table below)
*  - random colors (see table below)
*  - team-based colors (CT=blue, T=red of course)
*  - lots of comments in my code (the way its done)
*
* TODO:
*  - read default colors from a cfg or ini or something
*/


#include <amxconst>
#include <amxmod>
#include <amxmisc>
#include <string>

/* color tables. initialized at plugin_init() */
static weap_colors[31][3]
static rand_colors[23][3]

/* tables[playerid] store ammo count, last weapon id, and team name. */
new lastammo[33]
new lastweap[33]

/* cached beam sprite */
new spriteidx

/*	gamemode:
*	0 - disabled
*	1 - use nCOLOR vars for color (defaults to pink 255,0,204)
*	2 - random color from fixed table for every shot
*	3 - use weapon table for colors
*	4 - team based. CT=blue, T=red
*/
new gamemode

/* mode nomenclature */
static const modenames[5][] = { "disabled", "uniform", "random", "weapon-class", "player-team" }

/* nCOLOR: 0-255 - store rgb values. */
new nred
new ngreen
new nblue

public tracer_setmode(id, level, cid) {
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	new bool:nochange = false
	new args[1]

	read_argv(id, args, 1)

	if (equal(args,"0")) gamemode = 0
	else if (equal(args,"1")) gamemode = 1
	else if (equal(args,"2")) gamemode = 2
	else if (equal(args,"3")) gamemode = 3
	else if (equal(args,"4")) gamemode = 4
	else nochange = true

	console_print(id, "tracer mode is: %s", modenames[gamemode])

	// didn't change gamemode, don't display anything.
	if (nochange) {
		console_print(id, "usage: 'amx_tracers <0-4>' where: 0=off, 1=uniform, 2=random, 3=weapons, 4=teams")
		return PLUGIN_HANDLED
	}

	set_hudmessage(0, 0, 255, 0.05, 0.65, 1, 0.02, 10.0, 0.01, 0.1, 3)
	show_hudmessage(0, "tracer mode is: %s", modenames[gamemode])

	return PLUGIN_HANDLED
}


public tracer_setcolor(id, level, cid) {
	if (!cmd_access(id,level,cid,3))
		return PLUGIN_HANDLED

	new ared[4], agreen[4], ablue[4]
	new tnred, tngreen, tnblue

	// get args
	read_argv(1, ared, 3)
	read_argv(2, agreen, 3)
	read_argv(3, ablue, 3)

	// convert to int
	tnred = strtonum(ared)
	tngreen = strtonum(agreen)
	tnblue = strtonum(ablue)

	// check bounds
	if (tnred < 0) tnred = 0
	if (tngreen < 0) tngreen = 0
	if (tnblue < 0) tnblue = 0
	if (tnred > 255) tnred = 255
	if (tngreen > 255) tngreen = 255
	if (tnblue > 255) tnblue = 255

	// values ok, update application
	nred = tnred
	ngreen = tngreen
	nblue = tnblue

	return PLUGIN_HANDLED
}


public make_tracer(id) {

	if (gamemode == 0) return PLUGIN_CONTINUE

	new weap = read_data(2)		// id of the weapon
	new ammo = read_data(3)		// ammo left in clip
	new rval
	new pteam[16]

	get_user_team(id, pteam, 15)

	/* if no lastweap is set, you'll miss the first tracer!
	   weap is never zero.
	*/
	if (lastweap[id] == 0) { lastweap[id] = weap; }

	/* fire this event only if the ammo has changed but the weapon has not.
	   this prevents a tracer from being drawn when you switch from a weapon
	   with a larger clip to one with a smaller clip.

	   also, new ammo setting must be less.. otherwise a tracer is fired on reload.
	*/
	if ((lastammo[id] > ammo) && (lastweap[id] == weap)) {

		new vec1[3], vec2[3]
		get_user_origin(id, vec1, 1) // origin; where you are
		get_user_origin(id, vec2, 3) // termina; where your bullet goes

		message_begin(MSG_PAS, SVC_TEMPENTITY, vec1)
		write_byte(0)		// TE_BEAMPOINTS
		write_coord(vec1[0])	// start point
		write_coord(vec1[1])
		write_coord(vec1[2])
		write_coord(vec2[0])	// end point
		write_coord(vec2[1])
		write_coord(vec2[2])
		write_short(spriteidx)	// sprite to draw (precached below)
		write_byte(0)			// starting frame
		write_byte(0)			// frame rate
		write_byte(4)			// life in 0.1s
		write_byte(1)			// line width in 0.1u
		write_byte(0)			// noise in 0.1u

		switch (gamemode) {
			case 4: {
				if (equali(pteam,"CT")) write_byte(0); else write_byte(255)
				write_byte(0)
				if (equali(pteam,"CT")) write_byte(255); else write_byte(0)
			}
			case 3: {
				write_byte(weap_colors[weap][0])
				write_byte(weap_colors[weap][1])
				write_byte(weap_colors[weap][2])
			}
			case 2: {
				rval = random_num(0, 22)
				write_byte(rand_colors[rval][0])
				write_byte(rand_colors[rval][1])
				write_byte(rand_colors[rval][2])
			}
			default: {
				write_byte(nred)
				write_byte(ngreen)
				write_byte(nblue)
			}
		}

		write_byte(120)				// brightness
		write_byte(50)				// scroll speed
		message_end()
	}

	lastammo[id] = ammo
	lastweap[id] = weap

	return PLUGIN_CONTINUE
}

public plugin_init() {

	gamemode = 1
	nred = 255
	ngreen = 0
	nblue = 204

	rand_colors[0]  = {0, 255, 255}    // cyan
	rand_colors[1]  = {255, 0, 255}    // magenta

	rand_colors[2]  = {255, 255, 204}  // lt yellow
	rand_colors[3]  = {255, 255, 0}    // yellow
	rand_colors[4]  = {153, 153, 0}    // dark yellow/gold

	rand_colors[5]  = {204, 204, 255}  // lt blue
	rand_colors[6]  = {0, 0, 255}      // blue
	rand_colors[7]  = {0, 0, 102}      // dark blue

	rand_colors[8]  = {255, 204, 255}  // lt purple
	rand_colors[9]  = {204, 0, 204}    // purple
	rand_colors[10] = {102, 0, 102}    // dark purple

	rand_colors[11] = {255, 102, 102}  // lt red
	rand_colors[12] = {255, 0, 0}      // red
	rand_colors[13] = {102, 0, 0}      // dark red

	rand_colors[14] = {0, 255, 0}      // lt green
	rand_colors[15] = {0, 153, 0}      // green
	rand_colors[16] = {0, 102, 0}      // dark green

	rand_colors[17] = {255, 204, 153}  // lt orange
	rand_colors[18] = {255, 153, 0}    // orange
	rand_colors[19] = {153, 102, 0}    // brown
	rand_colors[20] = {102, 102, 102}  // gray
	rand_colors[21] = {204, 204, 204}  // lt gray
	rand_colors[22] = {255, 255, 255}  // white


	weap_colors[CSW_USP]		= { 0, 0, 255 } // blue
	weap_colors[CSW_GLOCK18]	= { 0, 0, 255 }
	weap_colors[CSW_P228]		= { 0, 0, 255 }
	weap_colors[CSW_ELITE]		= { 0, 0, 255 }
	weap_colors[CSW_FIVESEVEN]	= { 0, 0, 255 }
	weap_colors[CSW_DEAGLE]		= { 0, 0, 255 }

	weap_colors[CSW_XM1014]		= { 255, 255, 0 } // yellow
	weap_colors[CSW_M3]		= { 255, 255, 0 }

	weap_colors[CSW_MP5NAVY]	= { 255, 153, 0 } // orange
	weap_colors[CSW_TMP]		= { 255, 153, 0 }
	weap_colors[CSW_MAC10]		= { 255, 153, 0 }
	weap_colors[CSW_UMP45]		= { 255, 153, 0 }
	weap_colors[CSW_P90]		= { 255, 153, 0 }

	weap_colors[CSW_M4A1]	= { 102, 0, 0 } // dark red
	weap_colors[CSW_AUG]	= { 102, 0, 0 }
	weap_colors[CSW_SG552]	= { 102, 0, 0 }
	weap_colors[CSW_AK47]	= { 102, 0, 0 }
	weap_colors[CSW_G3SG1]	= { 102, 0, 0 }
	weap_colors[CSW_SG550]	= { 102, 0, 0 }

	weap_colors[CSW_SCOUT]	= { 102, 102, 102 } // lt gray
	weap_colors[CSW_AWP]	= { 204, 204, 204 } // gray

	weap_colors[CSW_M249]	= { 0, 255, 0 } // lt green


	register_concmd("amx_tracers", "tracer_setmode", ADMIN_LEVEL_B, "<0-4> - 0:off,1:uniform,2:random,3:weapons,4:teams")
	register_concmd("amx_tracers_colors", "tracer_setcolor", ADMIN_LEVEL_B, "<0-255> <0-255> <0-255> - RGB color values")
	register_plugin("tracer_fire","1.3","jon")
	register_event("CurWeapon","make_tracer","be","1=1","3>0")
	return PLUGIN_CONTINUE
}

public plugin_precache() {
	spriteidx = precache_model("sprites/laserbeam.spr");
	return PLUGIN_CONTINUE
}

