/***********************************************************************
 * amx_ejl_whoplayer.sma      version 1.3 (Dec 29,2002)  November 3/2002
 *  By:  Eric Lidman	Alias: Ludwig van       ejlmozart@hotmail.com 
 *  Upgrade: http://lidmanmusic.com/cs/plugins.html  
 *
 * This plugin serves 2 functions:
 *
 * 1. Find out if known players are playing on the server even if they
 *    are using a different name. Good for tracking regular players, 
 *    trouble-maker players, and suspected cheaters.
 * 2. Logs all players AUTHIDS to file with their names beside them.
 *    Using the log file makes evil player/cheater management easier.
 *    This was requested by the pixscotty server and they report to me
 *    its been extreemly useful. I myself have found it valuable. I
 *    hope it helps whoever gets it. *** Revised -- now includes IP
 *    adresses as well.  
 *
 *
 * Commands:
 *
 *  amx_whoplayer   -- shows whos playing, listing their known name
 *                       and matching it up with thier current name.
 *  amx_who_reload  -- If you make changes the the regplayers.txt and
 *                       you want to see them right away, issue this
 *                       command. Otherwise you will have to wait for the
 *                       map to change for the file to reload showing
 *                       the changes. Admin ACCESS_RCON is required.
 *  amx_who_add <index> <name in quotes>
 *                  -- To add players to the known list regplayers.txt 
 *                     in-game you can now use this command. You add 
 *                     them by their user index. To aquire these number,
 *                     type "status" in your console. I do not want to
 *                     add by name or part of name because there is too
 *                     much potential for inaccuracy. The first and 
 *                     shortest number listed in "status" for a player
 *                     is his index. Then, after the number, give them 
 *                     a name by which to recognize them later
 * 
 * Setup:  You will need to create a text file called: regplayers.txt
 *   It goes in your ..addons/amx directory.
 *   The format of it is: <wonid> <name>
 *
 *     example:       1234576 Ludwigvan
 *                    34343   vodka
 * 
 *   The name cannot contain spaces. This name is how you will identify
 *   a regular player. If there is no match for a players wonid, the name 
 *   will be *****, indicating an unknown player. Under its current
 *   setup, you can only have 200 entries in the file. 
 *   There were thoughts of making the collection of regular players
 *   automatic. Feel free to if you like of course. My thought was that
 *   any system I came up with to do that would make the server laggy
 *   using alot of CPU and memory and would thrash the hard-drive on
 *   client connects. So I opted to make the adding of regular players
 *   manual instead to avoid all that. It shouldnt be too hard to do if
 *   you use the info in the id log files this plugin produces.
 *
 *  Important: For the logging to work, you need to set up some folders
 *   on your server. You need: ../%moddir%/admin_logs/id_logs
 *
 *     example:      ../cstrike/admin_logs/id_logs
 * 
 **********************************************************************/

#include <amxmod>

new RegPlaInfo[500][39]
new RegPlaName[500][32]
new TotalRegPla
new OldName[33][32]
new On[33]

public plugin_init() 
{
	register_plugin("WhoPlayerIDTracking", "0.8", "EJL")
	register_clcmd("amx_whoplayer", "admin_whoplayer", ADMIN_USER, "amx_whoplayer  :  Find out what players are currently playing if have a different name.")
	register_clcmd("amx_who_reload", "admin_who_reload", ADMIN_RCON, "amx_who_reload : Reloads regplayers.txt in the middle of a round.")
	register_clcmd("amx_who_add", "admin_who_add", ADMIN_RCON, "amx_who_add <index> <name in quotes> : adds a player to regplayers.txt")
	if(file_exists("addons/amx/regplayers.txt") == 1){
		new line = 0
		new data[128]
		new stextsize = 0
		while((line=read_file("addons/amx/regplayers.txt",line,data,128,stextsize))!=0){
			if( (equal(data,"")) || (equal(data," ",1)) ){
				continue
			}else{
				TotalRegPla += 1
				if(TotalRegPla < 500){
					parse(data, RegPlaInfo[TotalRegPla], 39, RegPlaName[TotalRegPla], 32)
				}
			}
		}
	}

	new ThisMap[33]
	get_mapname(ThisMap,33)
	new idlog[192]
	new logtime[64]
	new filename[100]
	get_time("%m/%d/%Y  %H:%M",logtime,64)
	get_time("admin_logs/id_logs/%Y-%m-%d--id.log",filename,100)
	format(idlog,192,"%s  START MAP:  %s", logtime,ThisMap)
	write_file(filename, idlog, -1)

	return PLUGIN_CONTINUE
}

public admin_whoplayer(id){

	new i
	new j
	new maxpl = get_maxplayers()+1
	new Name[32]
	new Authid[39]
	new RegPlaCurName[32]
	new doneit

	client_print(id,print_console,"^nCurrent Name     /    Known Name + Cheater Status?")

	for (j = 1; j <= maxpl; j++) {
		doneit = 0
		get_user_authid(j,Authid,39)
		get_user_name(j,Name,32)
		for (i = 1; i <= TotalRegPla; i++) {
			if(is_user_connected(j) == 1){		
				if(equal(Authid, RegPlaInfo[i])){
					copy(RegPlaCurName, 39, Name)
					client_print(id,print_console, "%s            %s", RegPlaCurName,RegPlaName[i])
					doneit = 1
				}
			}
		}
		if(is_user_connected(j) == 1){
			if(doneit == 0){
				client_print(id,print_console, "%s            *****", Name)
			}
		}
	}
	client_cmd(id,"echo  ")

	return PLUGIN_HANDLED
}

public admin_who_reload(id){
	if (!(get_user_flags(id)&ADMIN_RCON)){
		client_print(id,print_console,"[AMX] You have no access to that command")
		return PLUGIN_HANDLED
	}
	TotalRegPla = 0

	if(file_exists("addons/amx/regplayers.txt") == 1){
		new line = 0
		new data[128]
		new stextsize = 0
		while((line=read_file("addons/amx/regplayers.txt",line,data,128,stextsize))!=0){
			if( (equal(data,"")) || (equal(data," ",1)) ){
				continue
			}else{
				TotalRegPla += 1
				if(TotalRegPla < 500){
					parse(data, RegPlaInfo[TotalRegPla], 39, RegPlaName[TotalRegPla], 32)
				}
			}
		}
		if(TotalRegPla >= 500)
			client_cmd(id,"echo Error: regplayers.txt is full at %d lines. Modify plugin to allow for more.",TotalRegPla)
		else
			client_cmd(id,"echo Reloaded: regplayers.txt.   Any changes made are now in effect.")
	}
	return PLUGIN_HANDLED
}

public admin_who_add(id){
	if (!(get_user_flags(id)&ADMIN_RCON)){
		client_print(id,print_console,"[AMX] You have no access to that command")
		return PLUGIN_HANDLED
	}
	if (read_argc() < 2){ 
		client_print(id,print_console,"[AMX] Usage: amx_who_add <# index> <name in quotes> : adds a player to regplayers.txt")
		return PLUGIN_HANDLED
	} 

	new fail = 0
	new sindex[4]
	new T_index
	new Authid[39]
	new Name[32]
	new commonname[32]
	read_argv(1,sindex,4)
	read_argv(2,commonname,32)
	T_index = strtonum(sindex)
	if( (T_index > 0) && (T_index < 33) ){
		if(is_user_connected(T_index) == 1){
			get_user_authid(T_index,Authid,39)
			get_user_name(T_index,Name,32)
		}else{
			client_print(id,print_console,"[AMX] FAIL: No player present for this index number.")
			return PLUGIN_HANDLED
		}	
	}else{
		client_print(id,print_console,"[AMX] FAIL: Player index invalid.")
		return PLUGIN_HANDLED
	}		
	if(strlen(commonname) > 30){
		client_print(id,print_console,"[AMX] Common identifier name must be 30 characters or less")
		return PLUGIN_HANDLED
	}
	if(strlen(commonname) == 0)
		copy(commonname, 30, Name)
	if(strlen(commonname) == 0){
		client_print(id,print_console,"[AMX] FAIL: No player name found for this index number.")
		return PLUGIN_HANDLED
	}			
	for (new i = 1; i <= TotalRegPla; i++) {
		if(equal(Authid, RegPlaInfo[i])){
			fail = 1
		}
	}
	if(fail == 0){
		new newentry[128]
		format(commonname,32,"^"%s^"",commonname)
		format(newentry,128,"%s  %s",Authid,commonname)
		write_file("addons/amx/regplayers.txt",newentry,-1)
		client_cmd(id,"echo [AMX] New entry added to file. Attempting to reload file.")
		admin_who_reload(id)
	}else{		
		client_cmd(id,"echo [AMX] FAIL: Target player is already in the regplayers.txt")
	}
	return PLUGIN_HANDLED
}

public client_connect(id){

	new User[32]
	new AUTHID[39]
	new IPAddy[32]
	new idlog[192]
	new logtime[64]
	new filename[100]
	get_time("%m/%d/%Y  %H:%M",logtime,64)
	get_time("admin_logs/id_logs/%Y-%m-%d--id.log",filename,100)

	get_user_authid(id,AUTHID,39)
	get_user_name(id,User,32)
	get_user_ip(id,IPAddy,32,1)
	copy(OldName[id],32,User)
	On[id] =1

	format(idlog, 192, "%s  %s   %s    %s",logtime, AUTHID,IPAddy, User)
	write_file(filename, idlog, -1)

	return PLUGIN_CONTINUE
}

public client_disconnect(id){
	On[id]=0
	return PLUGIN_CONTINUE
}

public client_infochanged(id){

	if(On[id] == 1){	
		new User[32]
		get_user_name(id,User,32)
		if(!equal(OldName[id],User)){
			new idlog[192]
			new logtime[64]
			new filename[100]
			new AUTHID[39]
			new IPAddy[32]
			get_time("%m/%d/%Y  %H:%M",logtime,64)
			get_time("admin_logs/id_logs/%Y-%m-%d--id.log",filename,100)
			get_user_authid(id,AUTHID,39)
			get_user_ip(id,IPAddy,32,1)
			format(idlog, 192, "%s  %s   %s    %s",logtime, AUTHID,IPAddy, User)
			write_file(filename, idlog, -1)
			copy(OldName[id],32,User)
		}
	}	
	return PLUGIN_CONTINUE
}


